
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

#if __ANDROID__
using Android.App;
using Android.Runtime;
#endif
using System.Runtime.InteropServices;

namespace com.amazon.whisperplay.fling.cpt.media.service
{
    public abstract partial class AmazonFlingPlayerServiceImpl
    {
#if __ANDROID__
        private class AmazonFlingPlayerServiceAndroid : AmazonFlingPlayerServiceDelegatesBase
        {
            private static AmazonFlingPlayerServiceAndroid instance;
            private static JValue androidActivity;
            
            private AmazonFlingPlayerServiceAndroid()
            {
            }
            
            public static AmazonFlingPlayerServiceAndroid Instance
            {
                get
                {
                    if (instance == null)
                    {
                        instance = new AmazonFlingPlayerServiceAndroid();
                        Java.Lang.JavaSystem.LoadLibrary("AmazonFlingPlayerServiceBridge");
                        instance.Start();
                    }
                    
                    return instance;
                }
            }
            
            [DllImport ("AmazonFlingPlayerServiceBridge")]
            private static extern string nativeRegisterCallback(CallbackDelegate callback);

            [DllImport ("AmazonFlingPlayerServiceBridge")]
            private static extern string nativeRegisterEventListener(CallbackDelegate callback);


            [DllImport ("AmazonFlingPlayerServiceBridge")]
            private static extern string nativeInit();

            [DllImport ("AmazonFlingPlayerServiceBridge")]
            private static extern string nativeStartServiceJson(string jsonMessage);

            [DllImport ("AmazonFlingPlayerServiceBridge")]
            private static extern string nativeUpdateVolumeJson(string jsonMessage);

            [DllImport ("AmazonFlingPlayerServiceBridge")]
            private static extern string nativeUpdateMuteJson(string jsonMessage);

            [DllImport ("AmazonFlingPlayerServiceBridge")]
            private static extern string nativeUpdatePositionJson(string jsonMessage);

            [DllImport ("AmazonFlingPlayerServiceBridge")]
            private static extern string nativeUpdateDurationJson(string jsonMessage);

            [DllImport ("AmazonFlingPlayerServiceBridge")]
            private static extern string nativeUpdateMediaStateJson(string jsonMessage);

            [DllImport ("AmazonFlingPlayerServiceBridge")]
            private static extern string nativeUpdateMediaConditionJson(string jsonMessage);

            [DllImport ("AmazonFlingPlayerServiceBridge")]
            private static extern string nativeAddSupportedMimeTypeJson(string jsonMessage);

            [DllImport ("AmazonFlingPlayerServiceBridge")]
            private static extern string nativeUpdateMediaPlayerInfoJson(string jsonMessage);


            protected override void NativeInit()
            {
                AmazonFlingPlayerServiceAndroid.nativeInit();
            }

            protected override void NativeRegisterCallback(CallbackDelegate callback)
            {
                AmazonFlingPlayerServiceAndroid.nativeRegisterCallback(callback);
            }

            protected override void NativeRegisterEventListener(CallbackDelegate callback)
            {
                AmazonFlingPlayerServiceAndroid.nativeRegisterEventListener(callback);
            }

            protected override void NativeRegisterCrossPlatformTool(string crossPlatformTool)
            {
            }

            protected override string NativeStartServiceJson(string jsonMessage)
            {
                return AmazonFlingPlayerServiceAndroid.nativeStartServiceJson(jsonMessage);
            }

            protected override string NativeUpdateVolumeJson(string jsonMessage)
            {
                return AmazonFlingPlayerServiceAndroid.nativeUpdateVolumeJson(jsonMessage);
            }

            protected override string NativeUpdateMuteJson(string jsonMessage)
            {
                return AmazonFlingPlayerServiceAndroid.nativeUpdateMuteJson(jsonMessage);
            }

            protected override string NativeUpdatePositionJson(string jsonMessage)
            {
                return AmazonFlingPlayerServiceAndroid.nativeUpdatePositionJson(jsonMessage);
            }

            protected override string NativeUpdateDurationJson(string jsonMessage)
            {
                return AmazonFlingPlayerServiceAndroid.nativeUpdateDurationJson(jsonMessage);
            }

            protected override string NativeUpdateMediaStateJson(string jsonMessage)
            {
                return AmazonFlingPlayerServiceAndroid.nativeUpdateMediaStateJson(jsonMessage);
            }

            protected override string NativeUpdateMediaConditionJson(string jsonMessage)
            {
                return AmazonFlingPlayerServiceAndroid.nativeUpdateMediaConditionJson(jsonMessage);
            }

            protected override string NativeAddSupportedMimeTypeJson(string jsonMessage)
            {
                return AmazonFlingPlayerServiceAndroid.nativeAddSupportedMimeTypeJson(jsonMessage);
            }

            protected override string NativeUpdateMediaPlayerInfoJson(string jsonMessage)
            {
                return AmazonFlingPlayerServiceAndroid.nativeUpdateMediaPlayerInfoJson(jsonMessage);
            }

            
            [DllImport ("AmazonFlingPlayerServiceBridge")]
            private static extern void nativeSetCurrentAndroidActivity(JValue activity);
            
            public override void SetCurrentAndroidActivity(Activity activity)
            {
                Start();
                AmazonFlingPlayerServiceAndroid.androidActivity = new JValue(activity);
                AmazonFlingPlayerServiceAndroid.nativeSetCurrentAndroidActivity(androidActivity);
            }
        }
#endif
    }
}

