
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/


#if __ANDROID__
using Android.App;
#endif
using System.Collections.Generic;
using System.Diagnostics;
using com.amazon.whisperplay.fling.cpt.media.service.json;

namespace com.amazon.whisperplay.fling.cpt.media.service 
{
    public abstract partial class AmazonFlingPlayerServiceImpl
    {
        private abstract class AmazonFlingPlayerServiceBase : AmazonFlingPlayerServiceImpl
        {
            static readonly System.Object startLock = new System.Object();
            static volatile bool startCalled = false;
        
            protected void Start () 
            {
                if(startCalled) 
                {
                    return;
                }
            
                lock(startLock)
                {
                    if(startCalled == false)
                    {
                        Init();
                        RegisterCallback();
                        RegisterEventListener();
                        RegisterCrossPlatformTool();
                        startCalled = true;
                    }
                }
            }

            protected abstract void Init();
            protected abstract void RegisterCallback();
            protected abstract void RegisterEventListener();
            protected abstract void RegisterCrossPlatformTool();
            
            public AmazonFlingPlayerServiceBase()
            {
                logger = new AmazonLogger(this.GetType().Name);
            }

            public override void UnityFireEvent(string jsonMessage)
            {
                AmazonFlingPlayerServiceImpl.FireEvent(jsonMessage);
            }
            public override void StartService(StartServiceParameters startServiceParameters)
            {
                Start();
                                Jsonable.CheckForErrors(Json.Deserialize(StartServiceJson(startServiceParameters.ToJson())) as Dictionary<string, object>);
            }

            private string StartServiceJson(string jsonMessage)
            {
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeStartServiceJson(jsonMessage);

                stopwatch.Stop();
                logger.Debug(string.Format("Successfully called native code in {0} ms", stopwatch.ElapsedMilliseconds));

                return result;
            }
        
            protected abstract string NativeStartServiceJson(string jsonMessage);

            public override void UpdateVolume(Volume volume)
            {
                Start();
                                Jsonable.CheckForErrors(Json.Deserialize(UpdateVolumeJson(volume.ToJson())) as Dictionary<string, object>);
            }

            private string UpdateVolumeJson(string jsonMessage)
            {
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeUpdateVolumeJson(jsonMessage);

                stopwatch.Stop();
                logger.Debug(string.Format("Successfully called native code in {0} ms", stopwatch.ElapsedMilliseconds));

                return result;
            }
        
            protected abstract string NativeUpdateVolumeJson(string jsonMessage);

            public override void UpdateMute(Mute mute)
            {
                Start();
                                Jsonable.CheckForErrors(Json.Deserialize(UpdateMuteJson(mute.ToJson())) as Dictionary<string, object>);
            }

            private string UpdateMuteJson(string jsonMessage)
            {
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeUpdateMuteJson(jsonMessage);

                stopwatch.Stop();
                logger.Debug(string.Format("Successfully called native code in {0} ms", stopwatch.ElapsedMilliseconds));

                return result;
            }
        
            protected abstract string NativeUpdateMuteJson(string jsonMessage);

            public override void UpdatePosition(Position position)
            {
                Start();
                                Jsonable.CheckForErrors(Json.Deserialize(UpdatePositionJson(position.ToJson())) as Dictionary<string, object>);
            }

            private string UpdatePositionJson(string jsonMessage)
            {
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeUpdatePositionJson(jsonMessage);

                stopwatch.Stop();
                logger.Debug(string.Format("Successfully called native code in {0} ms", stopwatch.ElapsedMilliseconds));

                return result;
            }
        
            protected abstract string NativeUpdatePositionJson(string jsonMessage);

            public override void UpdateDuration(Duration duration)
            {
                Start();
                                Jsonable.CheckForErrors(Json.Deserialize(UpdateDurationJson(duration.ToJson())) as Dictionary<string, object>);
            }

            private string UpdateDurationJson(string jsonMessage)
            {
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeUpdateDurationJson(jsonMessage);

                stopwatch.Stop();
                logger.Debug(string.Format("Successfully called native code in {0} ms", stopwatch.ElapsedMilliseconds));

                return result;
            }
        
            protected abstract string NativeUpdateDurationJson(string jsonMessage);

            public override void UpdateMediaState(MediaState mediaState)
            {
                Start();
                                Jsonable.CheckForErrors(Json.Deserialize(UpdateMediaStateJson(mediaState.ToJson())) as Dictionary<string, object>);
            }

            private string UpdateMediaStateJson(string jsonMessage)
            {
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeUpdateMediaStateJson(jsonMessage);

                stopwatch.Stop();
                logger.Debug(string.Format("Successfully called native code in {0} ms", stopwatch.ElapsedMilliseconds));

                return result;
            }
        
            protected abstract string NativeUpdateMediaStateJson(string jsonMessage);

            public override void UpdateMediaCondition(MediaCondition mediaCondition)
            {
                Start();
                                Jsonable.CheckForErrors(Json.Deserialize(UpdateMediaConditionJson(mediaCondition.ToJson())) as Dictionary<string, object>);
            }

            private string UpdateMediaConditionJson(string jsonMessage)
            {
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeUpdateMediaConditionJson(jsonMessage);

                stopwatch.Stop();
                logger.Debug(string.Format("Successfully called native code in {0} ms", stopwatch.ElapsedMilliseconds));

                return result;
            }
        
            protected abstract string NativeUpdateMediaConditionJson(string jsonMessage);

            public override void AddSupportedMimeType(MimeType mimeType)
            {
                Start();
                                Jsonable.CheckForErrors(Json.Deserialize(AddSupportedMimeTypeJson(mimeType.ToJson())) as Dictionary<string, object>);
            }

            private string AddSupportedMimeTypeJson(string jsonMessage)
            {
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeAddSupportedMimeTypeJson(jsonMessage);

                stopwatch.Stop();
                logger.Debug(string.Format("Successfully called native code in {0} ms", stopwatch.ElapsedMilliseconds));

                return result;
            }
        
            protected abstract string NativeAddSupportedMimeTypeJson(string jsonMessage);

            public override void UpdateMediaPlayerInfo(MediaPlayerInfo mediaPlayerInfo)
            {
                Start();
                                Jsonable.CheckForErrors(Json.Deserialize(UpdateMediaPlayerInfoJson(mediaPlayerInfo.ToJson())) as Dictionary<string, object>);
            }

            private string UpdateMediaPlayerInfoJson(string jsonMessage)
            {
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                string result = NativeUpdateMediaPlayerInfoJson(jsonMessage);

                stopwatch.Stop();
                logger.Debug(string.Format("Successfully called native code in {0} ms", stopwatch.ElapsedMilliseconds));

                return result;
            }
        
            protected abstract string NativeUpdateMediaPlayerInfoJson(string jsonMessage);


            public override void AddSetVolumeListener(SetVolumeDelegate responseDelegate)
            {
                Start();
                string eventId = "setVolume";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        eventListeners[eventId].Add(new SetVolumeDelegator (responseDelegate));
                    }
                    else 
                    {
                        var list = new List<IDelegator>();
                        list.Add(new SetVolumeDelegator(responseDelegate));
                        eventListeners.Add(eventId, list);
                    }
                }
            }
            
            public override void RemoveSetVolumeListener(SetVolumeDelegate responseDelegate)
            {
                Start();
                string eventId = "setVolume";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        foreach(SetVolumeDelegator delegator in eventListeners[eventId])
                        {
                            if(delegator.responseDelegate == responseDelegate)
                            {
                                eventListeners[eventId].Remove(delegator);
                                return;
                            }
                        }
                    }
                }
            }
            public override void AddSetMuteListener(SetMuteDelegate responseDelegate)
            {
                Start();
                string eventId = "setMute";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        eventListeners[eventId].Add(new SetMuteDelegator (responseDelegate));
                    }
                    else 
                    {
                        var list = new List<IDelegator>();
                        list.Add(new SetMuteDelegator(responseDelegate));
                        eventListeners.Add(eventId, list);
                    }
                }
            }
            
            public override void RemoveSetMuteListener(SetMuteDelegate responseDelegate)
            {
                Start();
                string eventId = "setMute";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        foreach(SetMuteDelegator delegator in eventListeners[eventId])
                        {
                            if(delegator.responseDelegate == responseDelegate)
                            {
                                eventListeners[eventId].Remove(delegator);
                                return;
                            }
                        }
                    }
                }
            }
            public override void AddPauseListener(PauseDelegate responseDelegate)
            {
                Start();
                string eventId = "pause";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        eventListeners[eventId].Add(new PauseDelegator (responseDelegate));
                    }
                    else 
                    {
                        var list = new List<IDelegator>();
                        list.Add(new PauseDelegator(responseDelegate));
                        eventListeners.Add(eventId, list);
                    }
                }
            }
            
            public override void RemovePauseListener(PauseDelegate responseDelegate)
            {
                Start();
                string eventId = "pause";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        foreach(PauseDelegator delegator in eventListeners[eventId])
                        {
                            if(delegator.responseDelegate == responseDelegate)
                            {
                                eventListeners[eventId].Remove(delegator);
                                return;
                            }
                        }
                    }
                }
            }
            public override void AddPlayListener(PlayDelegate responseDelegate)
            {
                Start();
                string eventId = "play";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        eventListeners[eventId].Add(new PlayDelegator (responseDelegate));
                    }
                    else 
                    {
                        var list = new List<IDelegator>();
                        list.Add(new PlayDelegator(responseDelegate));
                        eventListeners.Add(eventId, list);
                    }
                }
            }
            
            public override void RemovePlayListener(PlayDelegate responseDelegate)
            {
                Start();
                string eventId = "play";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        foreach(PlayDelegator delegator in eventListeners[eventId])
                        {
                            if(delegator.responseDelegate == responseDelegate)
                            {
                                eventListeners[eventId].Remove(delegator);
                                return;
                            }
                        }
                    }
                }
            }
            public override void AddStopListener(StopDelegate responseDelegate)
            {
                Start();
                string eventId = "stop";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        eventListeners[eventId].Add(new StopDelegator (responseDelegate));
                    }
                    else 
                    {
                        var list = new List<IDelegator>();
                        list.Add(new StopDelegator(responseDelegate));
                        eventListeners.Add(eventId, list);
                    }
                }
            }
            
            public override void RemoveStopListener(StopDelegate responseDelegate)
            {
                Start();
                string eventId = "stop";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        foreach(StopDelegator delegator in eventListeners[eventId])
                        {
                            if(delegator.responseDelegate == responseDelegate)
                            {
                                eventListeners[eventId].Remove(delegator);
                                return;
                            }
                        }
                    }
                }
            }
            public override void AddSeekListener(SeekDelegate responseDelegate)
            {
                Start();
                string eventId = "seek";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        eventListeners[eventId].Add(new SeekDelegator (responseDelegate));
                    }
                    else 
                    {
                        var list = new List<IDelegator>();
                        list.Add(new SeekDelegator(responseDelegate));
                        eventListeners.Add(eventId, list);
                    }
                }
            }
            
            public override void RemoveSeekListener(SeekDelegate responseDelegate)
            {
                Start();
                string eventId = "seek";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        foreach(SeekDelegator delegator in eventListeners[eventId])
                        {
                            if(delegator.responseDelegate == responseDelegate)
                            {
                                eventListeners[eventId].Remove(delegator);
                                return;
                            }
                        }
                    }
                }
            }
            public override void AddSetMediaSourceListener(SetMediaSourceDelegate responseDelegate)
            {
                Start();
                string eventId = "setMediaSource";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        eventListeners[eventId].Add(new SetMediaSourceDelegator (responseDelegate));
                    }
                    else 
                    {
                        var list = new List<IDelegator>();
                        list.Add(new SetMediaSourceDelegator(responseDelegate));
                        eventListeners.Add(eventId, list);
                    }
                }
            }
            
            public override void RemoveSetMediaSourceListener(SetMediaSourceDelegate responseDelegate)
            {
                Start();
                string eventId = "setMediaSource";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        foreach(SetMediaSourceDelegator delegator in eventListeners[eventId])
                        {
                            if(delegator.responseDelegate == responseDelegate)
                            {
                                eventListeners[eventId].Remove(delegator);
                                return;
                            }
                        }
                    }
                }
            }
            public override void AddSetPlayerStyleListener(SetPlayerStyleDelegate responseDelegate)
            {
                Start();
                string eventId = "setPlayerStyle";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        eventListeners[eventId].Add(new SetPlayerStyleDelegator (responseDelegate));
                    }
                    else 
                    {
                        var list = new List<IDelegator>();
                        list.Add(new SetPlayerStyleDelegator(responseDelegate));
                        eventListeners.Add(eventId, list);
                    }
                }
            }
            
            public override void RemoveSetPlayerStyleListener(SetPlayerStyleDelegate responseDelegate)
            {
                Start();
                string eventId = "setPlayerStyle";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        foreach(SetPlayerStyleDelegator delegator in eventListeners[eventId])
                        {
                            if(delegator.responseDelegate == responseDelegate)
                            {
                                eventListeners[eventId].Remove(delegator);
                                return;
                            }
                        }
                    }
                }
            }
            public override void AddSendCommandListener(SendCommandDelegate responseDelegate)
            {
                Start();
                string eventId = "sendCommand";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        eventListeners[eventId].Add(new SendCommandDelegator (responseDelegate));
                    }
                    else 
                    {
                        var list = new List<IDelegator>();
                        list.Add(new SendCommandDelegator(responseDelegate));
                        eventListeners.Add(eventId, list);
                    }
                }
            }
            
            public override void RemoveSendCommandListener(SendCommandDelegate responseDelegate)
            {
                Start();
                string eventId = "sendCommand";
                lock(eventLock)
                {
                    if (eventListeners.ContainsKey(eventId))
                    {
                        foreach(SendCommandDelegator delegator in eventListeners[eventId])
                        {
                            if(delegator.responseDelegate == responseDelegate)
                            {
                                eventListeners[eventId].Remove(delegator);
                                return;
                            }
                        }
                    }
                }
            }

#if __ANDROID__
            public override void SetCurrentAndroidActivity(Activity activity)
            {
                // do nothing
            }
#endif
        }
    }
}

