
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

#if __ANDROID__
using Android.App;
#endif
using System.Collections;

namespace com.amazon.whisperplay.fling.cpt.media.service 
{
    public delegate void SetVolumeDelegate(Volume volume);
    public delegate void SetMuteDelegate(Mute mute);
    public delegate void PauseDelegate();
    public delegate void PlayDelegate();
    public delegate void StopDelegate();
    public delegate void SeekDelegate(SeekParameters seekParameters);
    public delegate void SetMediaSourceDelegate(SetMediaSourceParameters setMediaSourceParameters);
    public delegate void SetPlayerStyleDelegate(PlayerStyle playerStyle);
    public delegate void SendCommandDelegate(Command command);

    public interface IAmazonFlingPlayerService
    {
        // AmazonFlingPlayerService API
        void StartService(StartServiceParameters startServiceParameters);
        void UpdateVolume(Volume volume);
        void UpdateMute(Mute mute);
        void UpdatePosition(Position position);
        void UpdateDuration(Duration duration);
        void UpdateMediaState(MediaState mediaState);
        void UpdateMediaCondition(MediaCondition mediaCondition);
        void AddSupportedMimeType(MimeType mimeType);
        void UpdateMediaPlayerInfo(MediaPlayerInfo mediaPlayerInfo);
        void UnityFireEvent(string jsonMessage);

        // Event API
        void AddSetVolumeListener (SetVolumeDelegate responseDelegate);
        void RemoveSetVolumeListener (SetVolumeDelegate responseDelegate);
        void AddSetMuteListener (SetMuteDelegate responseDelegate);
        void RemoveSetMuteListener (SetMuteDelegate responseDelegate);
        void AddPauseListener (PauseDelegate responseDelegate);
        void RemovePauseListener (PauseDelegate responseDelegate);
        void AddPlayListener (PlayDelegate responseDelegate);
        void RemovePlayListener (PlayDelegate responseDelegate);
        void AddStopListener (StopDelegate responseDelegate);
        void RemoveStopListener (StopDelegate responseDelegate);
        void AddSeekListener (SeekDelegate responseDelegate);
        void RemoveSeekListener (SeekDelegate responseDelegate);
        void AddSetMediaSourceListener (SetMediaSourceDelegate responseDelegate);
        void RemoveSetMediaSourceListener (SetMediaSourceDelegate responseDelegate);
        void AddSetPlayerStyleListener (SetPlayerStyleDelegate responseDelegate);
        void RemoveSetPlayerStyleListener (SetPlayerStyleDelegate responseDelegate);
        void AddSendCommandListener (SendCommandDelegate responseDelegate);
        void RemoveSendCommandListener (SendCommandDelegate responseDelegate);
#if __ANDROID__
        void SetCurrentAndroidActivity(Activity activity);
#endif
    }
}

