
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

#if __ANDROID__
using Android.App;
using Android.Runtime;
#endif
using System.Runtime.InteropServices;

namespace com.amazon.whisperplay.fling.cpt.media.controller
{
    public abstract partial class AmazonFlingServiceImpl
    {
#if __ANDROID__
        private class AmazonFlingServiceAndroid : AmazonFlingServiceDelegatesBase
        {
            private static AmazonFlingServiceAndroid instance;
            private static JValue androidActivity;
            
            private AmazonFlingServiceAndroid()
            {
            }
            
            public static AmazonFlingServiceAndroid Instance
            {
                get
                {
                    if (instance == null)
                    {
                        instance = new AmazonFlingServiceAndroid();
                        Java.Lang.JavaSystem.LoadLibrary("AmazonFlingServiceBridge");
                        instance.Start();
                    }
                    
                    return instance;
                }
            }
            
            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeRegisterCallback(CallbackDelegate callback);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeRegisterEventListener(CallbackDelegate callback);


            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeInit();

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeStartDiscoveryJson(string jsonMessage);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeStopDiscoveryJson(string jsonMessage);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeGetVolumeJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeSetVolumeJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeIsMuteJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeSetMuteJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeGetPositionJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeGetDurationJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeGetStatusJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeIsMimeTypeSupportedJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativePauseJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativePlayJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeStopJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeSeekJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeSetMediaSourceJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeSetPlayerStyleJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeAddStatusListenerJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeRemoveStatusListenerJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeSetPositionUpdateIntervalJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeSendCommandJson(string jsonMessage, string callerId);

            [DllImport ("AmazonFlingServiceBridge")]
            private static extern string nativeGetMediaInfoJson(string jsonMessage, string callerId);


            protected override void NativeInit()
            {
                AmazonFlingServiceAndroid.nativeInit();
            }

            protected override void NativeRegisterCallback(CallbackDelegate callback)
            {
                AmazonFlingServiceAndroid.nativeRegisterCallback(callback);
            }

            protected override void NativeRegisterEventListener(CallbackDelegate callback)
            {
                AmazonFlingServiceAndroid.nativeRegisterEventListener(callback);
            }

            protected override void NativeRegisterCrossPlatformTool(string crossPlatformTool)
            {
            }

            protected override string NativeStartDiscoveryJson(string jsonMessage)
            {
                return AmazonFlingServiceAndroid.nativeStartDiscoveryJson(jsonMessage);
            }

            protected override string NativeStopDiscoveryJson(string jsonMessage)
            {
                return AmazonFlingServiceAndroid.nativeStopDiscoveryJson(jsonMessage);
            }

            protected override string NativeGetVolumeJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeGetVolumeJson(jsonMessage, callerId);
            }

            protected override string NativeSetVolumeJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeSetVolumeJson(jsonMessage, callerId);
            }

            protected override string NativeIsMuteJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeIsMuteJson(jsonMessage, callerId);
            }

            protected override string NativeSetMuteJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeSetMuteJson(jsonMessage, callerId);
            }

            protected override string NativeGetPositionJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeGetPositionJson(jsonMessage, callerId);
            }

            protected override string NativeGetDurationJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeGetDurationJson(jsonMessage, callerId);
            }

            protected override string NativeGetStatusJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeGetStatusJson(jsonMessage, callerId);
            }

            protected override string NativeIsMimeTypeSupportedJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeIsMimeTypeSupportedJson(jsonMessage, callerId);
            }

            protected override string NativePauseJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativePauseJson(jsonMessage, callerId);
            }

            protected override string NativePlayJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativePlayJson(jsonMessage, callerId);
            }

            protected override string NativeStopJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeStopJson(jsonMessage, callerId);
            }

            protected override string NativeSeekJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeSeekJson(jsonMessage, callerId);
            }

            protected override string NativeSetMediaSourceJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeSetMediaSourceJson(jsonMessage, callerId);
            }

            protected override string NativeSetPlayerStyleJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeSetPlayerStyleJson(jsonMessage, callerId);
            }

            protected override string NativeAddStatusListenerJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeAddStatusListenerJson(jsonMessage, callerId);
            }

            protected override string NativeRemoveStatusListenerJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeRemoveStatusListenerJson(jsonMessage, callerId);
            }

            protected override string NativeSetPositionUpdateIntervalJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeSetPositionUpdateIntervalJson(jsonMessage, callerId);
            }

            protected override string NativeSendCommandJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeSendCommandJson(jsonMessage, callerId);
            }

            protected override string NativeGetMediaInfoJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceAndroid.nativeGetMediaInfoJson(jsonMessage, callerId);
            }

            
            [DllImport ("AmazonFlingServiceBridge")]
            private static extern void nativeSetCurrentAndroidActivity(JValue activity);
            
            public override void SetCurrentAndroidActivity(Activity activity)
            {
                Start();
                AmazonFlingServiceAndroid.androidActivity = new JValue(activity);
                AmazonFlingServiceAndroid.nativeSetCurrentAndroidActivity(androidActivity);
            }
        }
#endif
    }
}

