
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

using System.Runtime.InteropServices;

namespace com.amazon.whisperplay.fling.cpt.media.controller
{
    public abstract partial class AmazonFlingServiceImpl
    {
#if __IOS__
        private class AmazonFlingServiceIOS : AmazonFlingServiceDelegatesBase
        {
            private static AmazonLogger logger = new AmazonLogger("AmazonFlingServiceIOS");

            [DllImport ("__Internal")]
            private static extern string nativeRegisterCallback(CallbackDelegate callback);

            [DllImport ("__Internal")]
            private static extern string nativeRegisterEventListener(CallbackDelegate callback);

            [DllImport ("__Internal")]
            private static extern string nativeRegisterCrossPlatformTool(string crossPlatformTool);

            [DllImport ("__Internal")]
            private static extern string nativeStartDiscoveryJson(string jsonMessage);

            [DllImport ("__Internal")]
            private static extern string nativeStopDiscoveryJson(string jsonMessage);

            [DllImport ("__Internal")]
            private static extern string nativeGetVolumeJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeSetVolumeJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeIsMuteJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeSetMuteJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeGetPositionJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeGetDurationJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeGetStatusJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeIsMimeTypeSupportedJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativePauseJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativePlayJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeStopJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeSeekJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeSetMediaSourceJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeSetPlayerStyleJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeAddStatusListenerJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeRemoveStatusListenerJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeSetPositionUpdateIntervalJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeSendCommandJson(string jsonMessage, string callerId);

            [DllImport ("__Internal")]
            private static extern string nativeGetMediaInfoJson(string jsonMessage, string callerId);


            protected override void NativeInit()
            {
                //do nothing
            }

            protected override void NativeRegisterCallback(CallbackDelegate callback)
            {
                AmazonFlingServiceIOS.nativeRegisterCallback(callback);
            }

            protected override void NativeRegisterEventListener(CallbackDelegate callback)
            {
                AmazonFlingServiceIOS.nativeRegisterEventListener(callback);
            }

            protected override void NativeRegisterCrossPlatformTool(string crossPlatformTool)
            {
                AmazonFlingServiceIOS.nativeRegisterCrossPlatformTool(crossPlatformTool);
            }

            protected override string NativeStartDiscoveryJson(string jsonMessage)
            {
                return AmazonFlingServiceIOS.nativeStartDiscoveryJson(jsonMessage);
            }

            protected override string NativeStopDiscoveryJson(string jsonMessage)
            {
                return AmazonFlingServiceIOS.nativeStopDiscoveryJson(jsonMessage);
            }

            protected override string NativeGetVolumeJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeGetVolumeJson(jsonMessage, callerId);
            }

            protected override string NativeSetVolumeJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeSetVolumeJson(jsonMessage, callerId);
            }

            protected override string NativeIsMuteJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeIsMuteJson(jsonMessage, callerId);
            }

            protected override string NativeSetMuteJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeSetMuteJson(jsonMessage, callerId);
            }

            protected override string NativeGetPositionJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeGetPositionJson(jsonMessage, callerId);
            }

            protected override string NativeGetDurationJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeGetDurationJson(jsonMessage, callerId);
            }

            protected override string NativeGetStatusJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeGetStatusJson(jsonMessage, callerId);
            }

            protected override string NativeIsMimeTypeSupportedJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeIsMimeTypeSupportedJson(jsonMessage, callerId);
            }

            protected override string NativePauseJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativePauseJson(jsonMessage, callerId);
            }

            protected override string NativePlayJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativePlayJson(jsonMessage, callerId);
            }

            protected override string NativeStopJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeStopJson(jsonMessage, callerId);
            }

            protected override string NativeSeekJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeSeekJson(jsonMessage, callerId);
            }

            protected override string NativeSetMediaSourceJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeSetMediaSourceJson(jsonMessage, callerId);
            }

            protected override string NativeSetPlayerStyleJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeSetPlayerStyleJson(jsonMessage, callerId);
            }

            protected override string NativeAddStatusListenerJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeAddStatusListenerJson(jsonMessage, callerId);
            }

            protected override string NativeRemoveStatusListenerJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeRemoveStatusListenerJson(jsonMessage, callerId);
            }

            protected override string NativeSetPositionUpdateIntervalJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeSetPositionUpdateIntervalJson(jsonMessage, callerId);
            }

            protected override string NativeSendCommandJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeSendCommandJson(jsonMessage, callerId);
            }

            protected override string NativeGetMediaInfoJson(string jsonMessage, string callerId)
            {
                return AmazonFlingServiceIOS.nativeGetMediaInfoJson(jsonMessage, callerId);
            }

        }
#endif
    }
}
