
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/


using System.Collections;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Linq;
using com.amazon.whisperplay.fling.cpt.media.controller.json;


namespace com.amazon.whisperplay.fling.cpt.media.controller
{
    public sealed class DoubleResult : Jsonable
    {
        private static AmazonLogger logger = new AmazonLogger("Pi");

        public double Doubleresult{get;set;}
        
        public string ToJson()
        {
            try
            {
                Dictionary<string, object> toJson = this.GetObjectDictionary();
                return Json.Serialize(toJson);
            }
            catch(System.ApplicationException ex)
            {
                throw new AmazonException("Error encountered while Jsoning", ex);
            }
        }

        public override Dictionary<string, object> GetObjectDictionary() 
        {
            try 
            {
                Dictionary<string, object> objectDictionary = new Dictionary<string, object>();
                
                objectDictionary.Add("doubleresult", Doubleresult);
                return objectDictionary;
            } 
            catch(System.ApplicationException ex) 
            {
                throw new AmazonException("Error encountered while getting object dictionary", ex);
            }
        }

        public static DoubleResult CreateFromDictionary(Dictionary<string, object> jsonMap) 
        {
            try 
            {
                if (jsonMap == null)
                {
                    return null;
                }

                var request = new DoubleResult();
                
                
                if(jsonMap.ContainsKey("doubleresult")) 
                {
                    try
                    {
                        request.Doubleresult = (double) jsonMap["doubleresult"];
                    }
                    catch (System.InvalidCastException ex)
                    {
                        // json serialization will often convert real numbers to integers whenever possible (e.g., 0.0 will be serialized as 0)
                        logger.Debug ("InvalidCastException: Could not cast value to double. Attempting to cast to long instead." + ex);
                        request.Doubleresult = (double) System.Convert.ToInt32(jsonMap["doubleresult"]);
                    }
                }

                return request;
            } 
            catch (System.ApplicationException ex) 
            {
                throw new AmazonException("Error encountered while creating Object from dicionary", ex);
            }
        }

        public static DoubleResult CreateFromJson(string jsonMessage)
        {
            try 
            {
                Dictionary<string, object> jsonMap = Json.Deserialize(jsonMessage) as Dictionary<string, object>;
                Jsonable.CheckForErrors(jsonMap);
                return CreateFromDictionary(jsonMap);
            }
            catch(System.ApplicationException ex)
            {
                throw new AmazonException("Error encountered while UnJsoning", ex);
            }
        }
        

        public static Dictionary<string, DoubleResult> MapFromJson(Dictionary<string, object> jsonMap)
        {
            Dictionary<string, DoubleResult> result = new Dictionary<string, DoubleResult>();
            foreach (var entry in jsonMap)
            {
                DoubleResult value = CreateFromDictionary(entry.Value as Dictionary<string, object>);
                result.Add(entry.Key, value);
            }
            return result;
        }
        
        public static List<DoubleResult> ListFromJson(List<object> array)
        {
            List<DoubleResult> result = new List<DoubleResult>();
            foreach (var e in array)
            {
                result.Add(CreateFromDictionary(e as Dictionary<string, object>));
            }
            return result;
        }
    }
}
