
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

#if __ANDROID__
using Android.App;
#endif
using System.Collections;

namespace com.amazon.whisperplay.fling.cpt.media.controller 
{
    public delegate void VoidResultDelegate(VoidResult voidResult);
    public delegate void LongResultDelegate(LongResult longResult);
    public delegate void DoubleResultDelegate(DoubleResult doubleResult);
    public delegate void BooleanResultDelegate(BooleanResult booleanResult);
    public delegate void MuteParametersDelegate(MuteParameters muteParameters);
    public delegate void StatusDelegate(Status status);
    public delegate void PlayerInfoDelegate(PlayerInfo playerInfo);
    public delegate void AmazonExceptionDelegate(AmazonException e);
    public delegate void PlayerDiscoveredDelegate(Player player);
    public delegate void PlayerLostDelegate(Player player);
    public delegate void DiscoveryFailureDelegate();
    public delegate void OnStatusChangeDelegate(OnStatusChangeParameters onStatusChangeParameters);

    public interface IAmazonFlingService
    {
        // AmazonFlingService API
        void StartDiscovery(PlayerServiceId playerServiceId);
        void StopDiscovery();
        void GetVolume(Player player, DoubleResultDelegate doubleResultDelegate, AmazonExceptionDelegate errorDelegate);
        void SetVolume(SetVolumeParameters setVolumeParameters, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        void IsMute(Player player, BooleanResultDelegate booleanResultDelegate, AmazonExceptionDelegate errorDelegate);
        void SetMute(MuteParameters muteParameters, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        void GetPosition(Player player, LongResultDelegate longResultDelegate, AmazonExceptionDelegate errorDelegate);
        void GetDuration(Player player, LongResultDelegate longResultDelegate, AmazonExceptionDelegate errorDelegate);
        void GetStatus(Player player, StatusDelegate statusDelegate, AmazonExceptionDelegate errorDelegate);
        void IsMimeTypeSupported(IsMimeTypeSupportedParameters isMimeTypeSupportedParameters, BooleanResultDelegate booleanResultDelegate, AmazonExceptionDelegate errorDelegate);
        void Pause(Player player, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        void Play(Player player, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        void Stop(Player player, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        void Seek(PlayerSeekParameters playerSeekParameters, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        void SetMediaSource(SetMediaSourceParameters setMediaSourceParameters, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        void SetPlayerStyle(SetPlayerStyleParameters setPlayerStyleParameters, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        void AddStatusListener(Player player, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        void RemoveStatusListener(Player player, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        void SetPositionUpdateInterval(SetPositionUpdateIntervalParameters setPositionUpdateIntervalParameters, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        void SendCommand(SendCommandParameters sendCommandParameters, VoidResultDelegate voidResultDelegate, AmazonExceptionDelegate errorDelegate);
        void GetMediaInfo(Player player, PlayerInfoDelegate playerInfoDelegate, AmazonExceptionDelegate errorDelegate);
        // Method to communicate events to the application. This method is usually called by the native side, via the amazon bridge.
        void UnityCallback(string jsonMessage);
        void UnityFireEvent(string jsonMessage);

        // Event API
        void AddPlayerDiscoveredListener (PlayerDiscoveredDelegate responseDelegate);
        void RemovePlayerDiscoveredListener (PlayerDiscoveredDelegate responseDelegate);
        void AddPlayerLostListener (PlayerLostDelegate responseDelegate);
        void RemovePlayerLostListener (PlayerLostDelegate responseDelegate);
        void AddDiscoveryFailureListener (DiscoveryFailureDelegate responseDelegate);
        void RemoveDiscoveryFailureListener (DiscoveryFailureDelegate responseDelegate);
        void AddOnStatusChangeListener (OnStatusChangeDelegate responseDelegate);
        void RemoveOnStatusChangeListener (OnStatusChangeDelegate responseDelegate);
#if __ANDROID__
        void SetCurrentAndroidActivity(Activity activity);
#endif
    }
}

