
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/


using System.Collections;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Linq;
using com.amazon.whisperplay.fling.cpt.media.controller.json;


namespace com.amazon.whisperplay.fling.cpt.media.controller
{
    public sealed class MuteParameters : Jsonable
    {
        private static AmazonLogger logger = new AmazonLogger("Pi");

        public Player Player{get;set;}
        public bool Mute{get;set;}
        
        public string ToJson()
        {
            try
            {
                Dictionary<string, object> toJson = this.GetObjectDictionary();
                return Json.Serialize(toJson);
            }
            catch(System.ApplicationException ex)
            {
                throw new AmazonException("Error encountered while Jsoning", ex);
            }
        }

        public override Dictionary<string, object> GetObjectDictionary() 
        {
            try 
            {
                Dictionary<string, object> objectDictionary = new Dictionary<string, object>();
                
                objectDictionary.Add("player", (Player != null) ? Player.GetObjectDictionary() : null);
                objectDictionary.Add("mute", Mute);
                return objectDictionary;
            } 
            catch(System.ApplicationException ex) 
            {
                throw new AmazonException("Error encountered while getting object dictionary", ex);
            }
        }

        public static MuteParameters CreateFromDictionary(Dictionary<string, object> jsonMap) 
        {
            try 
            {
                if (jsonMap == null)
                {
                    return null;
                }

                var request = new MuteParameters();
                
                
                if(jsonMap.ContainsKey("player")) 
                {
                    request.Player = Player.CreateFromDictionary(jsonMap["player"] as Dictionary<string, object>); 
                }
                
                if(jsonMap.ContainsKey("mute")) 
                {
                    request.Mute = (bool) jsonMap["mute"];
                }

                return request;
            } 
            catch (System.ApplicationException ex) 
            {
                throw new AmazonException("Error encountered while creating Object from dicionary", ex);
            }
        }

        public static MuteParameters CreateFromJson(string jsonMessage)
        {
            try 
            {
                Dictionary<string, object> jsonMap = Json.Deserialize(jsonMessage) as Dictionary<string, object>;
                Jsonable.CheckForErrors(jsonMap);
                return CreateFromDictionary(jsonMap);
            }
            catch(System.ApplicationException ex)
            {
                throw new AmazonException("Error encountered while UnJsoning", ex);
            }
        }
        

        public static Dictionary<string, MuteParameters> MapFromJson(Dictionary<string, object> jsonMap)
        {
            Dictionary<string, MuteParameters> result = new Dictionary<string, MuteParameters>();
            foreach (var entry in jsonMap)
            {
                MuteParameters value = CreateFromDictionary(entry.Value as Dictionary<string, object>);
                result.Add(entry.Key, value);
            }
            return result;
        }
        
        public static List<MuteParameters> ListFromJson(List<object> array)
        {
            List<MuteParameters> result = new List<MuteParameters>();
            foreach (var e in array)
            {
                result.Add(CreateFromDictionary(e as Dictionary<string, object>));
            }
            return result;
        }
    }
}
