
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/


using System.Collections;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Linq;
using com.amazon.whisperplay.fling.cpt.media.controller.json;


namespace com.amazon.whisperplay.fling.cpt.media.controller
{
    public sealed class Status : Jsonable
    {
        private static AmazonLogger logger = new AmazonLogger("Pi");

        public State State{get;set;}
        public Condition Condition{get;set;}
        public bool MuteSet{get;set;}
                public bool Mute{get;set;}
                public bool VolumeSet{get;set;}
                public double Volume{get;set;}
        
        public string ToJson()
        {
            try
            {
                Dictionary<string, object> toJson = this.GetObjectDictionary();
                return Json.Serialize(toJson);
            }
            catch(System.ApplicationException ex)
            {
                throw new AmazonException("Error encountered while Jsoning", ex);
            }
        }

        public override Dictionary<string, object> GetObjectDictionary() 
        {
            try 
            {
                Dictionary<string, object> objectDictionary = new Dictionary<string, object>();
                
                objectDictionary.Add("state", State);
                objectDictionary.Add("condition", Condition);
                objectDictionary.Add("muteSet", MuteSet);
                objectDictionary.Add("mute", Mute);
                objectDictionary.Add("volumeSet", VolumeSet);
                objectDictionary.Add("volume", Volume);
                return objectDictionary;
            } 
            catch(System.ApplicationException ex) 
            {
                throw new AmazonException("Error encountered while getting object dictionary", ex);
            }
        }

        public static Status CreateFromDictionary(Dictionary<string, object> jsonMap) 
        {
            try 
            {
                if (jsonMap == null)
                {
                    return null;
                }

                var request = new Status();
                
                
                if(jsonMap.ContainsKey("state")) 
                {
                    request.State = (State) System.Enum.Parse(typeof(State), (string) jsonMap["state"]);
                }
                
                if(jsonMap.ContainsKey("condition")) 
                {
                    request.Condition = (Condition) System.Enum.Parse(typeof(Condition), (string) jsonMap["condition"]);
                }
                
                if(jsonMap.ContainsKey("muteSet")) 
                {
                    request.MuteSet = (bool) jsonMap["muteSet"];
                }
                
                if(jsonMap.ContainsKey("mute")) 
                {
                    request.Mute = (bool) jsonMap["mute"];
                }
                
                if(jsonMap.ContainsKey("volumeSet")) 
                {
                    request.VolumeSet = (bool) jsonMap["volumeSet"];
                }
                
                if(jsonMap.ContainsKey("volume")) 
                {
                    try
                    {
                        request.Volume = (double) jsonMap["volume"];
                    }
                    catch (System.InvalidCastException ex)
                    {
                        // json serialization will often convert real numbers to integers whenever possible (e.g., 0.0 will be serialized as 0)
                        logger.Debug ("InvalidCastException: Could not cast value to double. Attempting to cast to long instead." + ex);
                        request.Volume = (double) System.Convert.ToInt32(jsonMap["volume"]);
                    }
                }

                return request;
            } 
            catch (System.ApplicationException ex) 
            {
                throw new AmazonException("Error encountered while creating Object from dicionary", ex);
            }
        }

        public static Status CreateFromJson(string jsonMessage)
        {
            try 
            {
                Dictionary<string, object> jsonMap = Json.Deserialize(jsonMessage) as Dictionary<string, object>;
                Jsonable.CheckForErrors(jsonMap);
                return CreateFromDictionary(jsonMap);
            }
            catch(System.ApplicationException ex)
            {
                throw new AmazonException("Error encountered while UnJsoning", ex);
            }
        }
        

        public static Dictionary<string, Status> MapFromJson(Dictionary<string, object> jsonMap)
        {
            Dictionary<string, Status> result = new Dictionary<string, Status>();
            foreach (var entry in jsonMap)
            {
                Status value = CreateFromDictionary(entry.Value as Dictionary<string, object>);
                result.Add(entry.Key, value);
            }
            return result;
        }
        
        public static List<Status> ListFromJson(List<object> array)
        {
            List<Status> result = new List<Status>();
            foreach (var e in array)
            {
                result.Add(CreateFromDictionary(e as Dictionary<string, object>));
            }
            return result;
        }
    }
}
