
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

//
//  AMAZONAmazonFlingService.h
//

#import <Foundation/Foundation.h>
#import "AMAZONPlayerServiceId.h"
#import "AMAZONDoubleResult.h"
#import "AMAZONPlayer.h"
#import "AMAZONVoidResult.h"
#import "AMAZONSetVolumeParameters.h"
#import "AMAZONBooleanResult.h"
#import "AMAZONMuteParameters.h"
#import "AMAZONLongResult.h"
#import "AMAZONStatus.h"
#import "AMAZONIsMimeTypeSupportedParameters.h"
#import "AMAZONPlayerSeekParameters.h"
#import "AMAZONSetMediaSourceParameters.h"
#import "AMAZONSetPlayerStyleParameters.h"
#import "AMAZONSetPositionUpdateIntervalParameters.h"
#import "AMAZONSendCommandParameters.h"
#import "AMAZONPlayerInfo.h"

@protocol AMAZONAmazonFlingServiceObjectiveCController;

typedef void (^futureDoubleResultBlock)(AMAZONDoubleResult * response, NSError * error);
typedef void (^futureVoidResultBlock)(AMAZONVoidResult * response, NSError * error);
typedef void (^futureBooleanResultBlock)(AMAZONBooleanResult * response, NSError * error);
typedef void (^futureVoidResultBlock)(AMAZONVoidResult * response, NSError * error);
typedef void (^futureLongResultBlock)(AMAZONLongResult * response, NSError * error);
typedef void (^futureLongResultBlock)(AMAZONLongResult * response, NSError * error);
typedef void (^futureStatusBlock)(AMAZONStatus * response, NSError * error);
typedef void (^futureBooleanResultBlock)(AMAZONBooleanResult * response, NSError * error);
typedef void (^futureVoidResultBlock)(AMAZONVoidResult * response, NSError * error);
typedef void (^futureVoidResultBlock)(AMAZONVoidResult * response, NSError * error);
typedef void (^futureVoidResultBlock)(AMAZONVoidResult * response, NSError * error);
typedef void (^futureVoidResultBlock)(AMAZONVoidResult * response, NSError * error);
typedef void (^futureVoidResultBlock)(AMAZONVoidResult * response, NSError * error);
typedef void (^futureVoidResultBlock)(AMAZONVoidResult * response, NSError * error);
typedef void (^futureVoidResultBlock)(AMAZONVoidResult * response, NSError * error);
typedef void (^futureVoidResultBlock)(AMAZONVoidResult * response, NSError * error);
typedef void (^futureVoidResultBlock)(AMAZONVoidResult * response, NSError * error);
typedef void (^futureVoidResultBlock)(AMAZONVoidResult * response, NSError * error);
typedef void (^futurePlayerInfoBlock)(AMAZONPlayerInfo * response, NSError * error);

@protocol AMAZONAmazonFlingService <NSObject>

@property (atomic, strong) id<AMAZONAmazonFlingServiceObjectiveCController> controller;

- (void)startDiscovery:(AMAZONPlayerServiceId *)playerServiceId error:(NSError *__autoreleasing *)errorPtr;
- (void)stopDiscoveryWithError:(NSError *__autoreleasing *)errorPtr;
- (void)getVolume:(AMAZONPlayer *)player completion:(futureDoubleResultBlock)completion;
- (void)setVolume:(AMAZONSetVolumeParameters *)setVolumeParameters completion:(futureVoidResultBlock)completion;
- (void)isMute:(AMAZONPlayer *)player completion:(futureBooleanResultBlock)completion;
- (void)setMute:(AMAZONMuteParameters *)muteParameters completion:(futureVoidResultBlock)completion;
- (void)getPosition:(AMAZONPlayer *)player completion:(futureLongResultBlock)completion;
- (void)getDuration:(AMAZONPlayer *)player completion:(futureLongResultBlock)completion;
- (void)getStatus:(AMAZONPlayer *)player completion:(futureStatusBlock)completion;
- (void)isMimeTypeSupported:(AMAZONIsMimeTypeSupportedParameters *)isMimeTypeSupportedParameters completion:(futureBooleanResultBlock)completion;
- (void)pause:(AMAZONPlayer *)player completion:(futureVoidResultBlock)completion;
- (void)play:(AMAZONPlayer *)player completion:(futureVoidResultBlock)completion;
- (void)stop:(AMAZONPlayer *)player completion:(futureVoidResultBlock)completion;
- (void)seek:(AMAZONPlayerSeekParameters *)playerSeekParameters completion:(futureVoidResultBlock)completion;
- (void)setMediaSource:(AMAZONSetMediaSourceParameters *)setMediaSourceParameters completion:(futureVoidResultBlock)completion;
- (void)setPlayerStyle:(AMAZONSetPlayerStyleParameters *)setPlayerStyleParameters completion:(futureVoidResultBlock)completion;
- (void)addStatusListener:(AMAZONPlayer *)player completion:(futureVoidResultBlock)completion;
- (void)removeStatusListener:(AMAZONPlayer *)player completion:(futureVoidResultBlock)completion;
- (void)setPositionUpdateInterval:(AMAZONSetPositionUpdateIntervalParameters *)setPositionUpdateIntervalParameters completion:(futureVoidResultBlock)completion;
- (void)sendCommand:(AMAZONSendCommandParameters *)sendCommandParameters completion:(futureVoidResultBlock)completion;
- (void)getMediaInfo:(AMAZONPlayer *)player completion:(futurePlayerInfoBlock)completion;

@end

