
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

//
//  AMAZONAmazonFlingServiceBridge.mm
//


#import "AMAZONAmazonFlingServiceBridge.h"
#import "AMAZONAmazonFlingServiceObjectiveCControllerImpl.h"

@implementation AMAZONAmazonFlingServiceBridge

- (instancetype)init
{
    self = [super init];
    if (self) {

    }
    return self;
}

// Converts C style string to NSString
NSString* CreateNSString (const char* string)
{
    if (string)
        return [NSString stringWithUTF8String: string];
    else
        return [NSString stringWithUTF8String: ""];
}

// Helper method to create C string copy
char* MakeStringCopy (const char* string)
{
    if (string == NULL)
        return NULL;
    
    char* res = (char*)malloc(strlen(string) + 1);
    strcpy(res, string);
    return res;
}

extern "C" {
    char* gameObjectName;
    char* crossPlatformToolName;
    
    //callback mechanism
    typedef void (*callback_delegate)(const char*);
    static callback_delegate callbackApiMethod;
    static const char* VOID_JSON = "{}";
    
    static char* voidResult() {
        char* result = strdup(VOID_JSON); //memory will be deallocated by managed code
        return result; //no errors
    }
    
    char* nativeRegisterEventListener(callback_delegate callback) {
        [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] setEventApiMethod:callback];
        return voidResult();
    }
    
    char* nativeRegisterCallbackGameObject(char* name) {
        gameObjectName = MakeStringCopy(name);
        [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] setCallbackGameObject:CreateNSString(name)];
        
        return MakeStringCopy(gameObjectName);
    }
    
    char* nativeRegisterCrossPlatformTool(char* name) {
        crossPlatformToolName = MakeStringCopy(name);
        [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] setCrossPlatformTool:CreateNSString(name)];
        
        return voidResult();
    }

    char* nativeStartDiscoveryJson (const char* playerServiceId) {
        NSData* data = [CreateNSString(playerServiceId) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] startDiscovery:data];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeStopDiscoveryJson (const char* voidArgument) {
        NSData* data = [CreateNSString(voidArgument) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] stopDiscovery:data];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeGetVolumeJson (const char* player, const char* uuid) {
        NSData* data = [CreateNSString(player) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] getVolume:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeSetVolumeJson (const char* setVolumeParameters, const char* uuid) {
        NSData* data = [CreateNSString(setVolumeParameters) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] setVolume:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeIsMuteJson (const char* player, const char* uuid) {
        NSData* data = [CreateNSString(player) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] isMute:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeSetMuteJson (const char* muteParameters, const char* uuid) {
        NSData* data = [CreateNSString(muteParameters) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] setMute:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeGetPositionJson (const char* player, const char* uuid) {
        NSData* data = [CreateNSString(player) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] getPosition:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeGetDurationJson (const char* player, const char* uuid) {
        NSData* data = [CreateNSString(player) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] getDuration:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeGetStatusJson (const char* player, const char* uuid) {
        NSData* data = [CreateNSString(player) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] getStatus:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeIsMimeTypeSupportedJson (const char* isMimeTypeSupportedParameters, const char* uuid) {
        NSData* data = [CreateNSString(isMimeTypeSupportedParameters) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] isMimeTypeSupported:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativePauseJson (const char* player, const char* uuid) {
        NSData* data = [CreateNSString(player) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] pause:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativePlayJson (const char* player, const char* uuid) {
        NSData* data = [CreateNSString(player) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] play:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeStopJson (const char* player, const char* uuid) {
        NSData* data = [CreateNSString(player) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] stop:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeSeekJson (const char* playerSeekParameters, const char* uuid) {
        NSData* data = [CreateNSString(playerSeekParameters) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] seek:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeSetMediaSourceJson (const char* setMediaSourceParameters, const char* uuid) {
        NSData* data = [CreateNSString(setMediaSourceParameters) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] setMediaSource:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeSetPlayerStyleJson (const char* setPlayerStyleParameters, const char* uuid) {
        NSData* data = [CreateNSString(setPlayerStyleParameters) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] setPlayerStyle:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeAddStatusListenerJson (const char* player, const char* uuid) {
        NSData* data = [CreateNSString(player) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] addStatusListener:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeRemoveStatusListenerJson (const char* player, const char* uuid) {
        NSData* data = [CreateNSString(player) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] removeStatusListener:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeSetPositionUpdateIntervalJson (const char* setPositionUpdateIntervalParameters, const char* uuid) {
        NSData* data = [CreateNSString(setPositionUpdateIntervalParameters) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] setPositionUpdateInterval:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeSendCommandJson (const char* sendCommandParameters, const char* uuid) {
        NSData* data = [CreateNSString(sendCommandParameters) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] sendCommand:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }

    char* nativeGetMediaInfoJson (const char* player, const char* uuid) {
        NSData* data = [CreateNSString(player) dataUsingEncoding:NSUTF8StringEncoding];
        NSData* returnJson = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl sharedInstance] getMediaInfo:data uuid:CreateNSString(uuid) completion:^(NSData* jsonData)
        {
            NSString* ret = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
#ifdef UNITY_VERSION
            UnitySendMessage(gameObjectName, "UnityCallback", MakeStringCopy([ret UTF8String]));
#else
            callbackApiMethod(MakeStringCopy([ret UTF8String]));
#endif
        }];
        
        NSString* returnString = [[NSString alloc] initWithData:returnJson encoding:NSUTF8StringEncoding];
        return MakeStringCopy([returnString UTF8String]);
    }


    const char* nativeRegisterCallback(callback_delegate callback) {
        callbackApiMethod = callback;
        return voidResult();
    }
    
    extern void UnitySendMessage(const char *, const char *, const char *);
}

@end
