
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

//
//  AMAZONAmazonFlingServiceJsonFromObject.m
//


#import "AMAZONAmazonFlingServiceJsonFromObject.h"
#import "AMAZONDoubleResult.h"
#import "AMAZONVoidResult.h"
#import "AMAZONBooleanResult.h"
#import "AMAZONLongResult.h"
#import "AMAZONStatus.h"
#import "AMAZONPlayerInfo.h"
#import "AMAZONPlayer.h"
#import "AMAZONOnStatusChangeParameters.h"

@interface AMAZONAmazonFlingServiceJsonFromObject ()

+ (NSArray *)jsonArrayFromNSArray:(NSArray *)arrayValue;
+ (NSDictionary *)jsonDictionaryFromNSDictionary:(NSDictionary *)dictValue;

@end

@implementation AMAZONAmazonFlingServiceJsonFromObject

+ (NSArray *)jsonArrayFromNSArray:(NSArray *)arrayValue {
    NSMutableArray* returnArray = [NSMutableArray array];
    for (id obj in arrayValue) {
        if([obj respondsToSelector:@selector(dictionaryValue)]) {
            NSDictionary* objDict = [obj performSelector:@selector(dictionaryValue)];
            [returnArray addObject:objDict];
        } else {
            [returnArray addObject:obj];
        }
    }
    return returnArray;
}

+ (NSDictionary *)jsonDictionaryFromNSDictionary:(NSDictionary *)dictValue {
    NSMutableDictionary* returnDictionary = [NSMutableDictionary dictionary];
    for (id key in dictValue) {
        id keyToUse = key;
        id valueToUse = [dictValue objectForKey:key];
        if([keyToUse respondsToSelector:@selector(dictionaryValue)]) {
            NSDictionary* keyDict = [keyToUse performSelector:@selector(dictionaryValue)];
            keyToUse = keyDict;
        }
        if([valueToUse respondsToSelector:@selector(dictionaryValue)]) {
            if([valueToUse respondsToSelector:@selector(dictionaryValue)]) {
                NSDictionary* objDict = [valueToUse performSelector:@selector(dictionaryValue)];
                valueToUse = objDict;
            }
        }
        [returnDictionary setObject:valueToUse forKey:keyToUse];
    }
    return returnDictionary;
}

+ (NSData *)jsonFromVoid {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    NSError *error;
    return [NSJSONSerialization dataWithJSONObject:returnDictionary options:0 error:&error];
}

+ (NSData *)jsonFromError:(NSError *)error {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    [returnDictionary setObject:[error localizedDescription] forKey:@"error"];
    NSError *jsonSerializerError;
    return [NSJSONSerialization dataWithJSONObject:returnDictionary options:0 error:&jsonSerializerError];
}

+ (NSData *)jsonFromFutureError:(NSError *)error uuid:(NSString *)uuid {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    NSMutableDictionary* errorDictionary = [[NSMutableDictionary alloc] init];
    [errorDictionary setObject:[error localizedDescription] forKey:@"error"];
    [returnDictionary setObject:errorDictionary forKey:@"response"];
    [returnDictionary setObject:uuid forKey:@"callerId"];
    NSError *jsonSerializerError;
    return [NSJSONSerialization dataWithJSONObject:returnDictionary options:0 error:&jsonSerializerError];
}

+ (NSData *)jsonFromEventError:(NSError *)error eventId:(NSString *)eventId {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    NSMutableDictionary* errorDictionary = [[NSMutableDictionary alloc] init];
    [errorDictionary setObject:[error localizedDescription] forKey:@"error"];
    [returnDictionary setObject:errorDictionary forKey:@"response"];
    [returnDictionary setObject:eventId forKey:@"eventId"];
    NSError *jsonSerializerError;
    return [NSJSONSerialization dataWithJSONObject:returnDictionary options:0 error:&jsonSerializerError];
}

+ (NSData *)jsonFromEventNoOutput:(NSString *)eventId {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    [returnDictionary setObject:[NSDictionary dictionary] forKey:@"response"];
    [returnDictionary setObject:eventId forKey:@"eventId"];
    NSError *jsonSerializerError;
    return [NSJSONSerialization dataWithJSONObject:returnDictionary options:0 error:&jsonSerializerError];
}

+ (NSData *)jsonFromDoubleResult:(AMAZONDoubleResult *)doubleResult uuid:(NSString *)uuid error:(NSError *__autoreleasing *)errorPtr {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    [returnDictionary setObject:uuid forKey:@"callerId"];
    NSDictionary* ret = [doubleResult dictionaryValue];
    if(![NSJSONSerialization isValidJSONObject:ret]) {
        if(errorPtr) {
            NSString * domain = @"com.amazon.TestingService.ErrorDomain";
            NSUInteger errorCode = 0;
            NSMutableDictionary * userInfo = [NSMutableDictionary dictionary];
            [userInfo setObject:@"An error occured in jsonFromDoubleResult: the object passed in is not a valid json serializable object." forKey:NSLocalizedDescriptionKey];
            *errorPtr = [NSError errorWithDomain:domain code:errorCode userInfo:userInfo];
        }
        return nil;
    }
    [returnDictionary setObject:ret forKey:@"response"];
    return [NSJSONSerialization dataWithJSONObject:returnDictionary options:0 error:errorPtr];
}

+ (NSData *)jsonFromVoidResult:(AMAZONVoidResult *)voidResult uuid:(NSString *)uuid error:(NSError *__autoreleasing *)errorPtr {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    [returnDictionary setObject:uuid forKey:@"callerId"];
    NSDictionary* ret = [voidResult dictionaryValue];
    if(![NSJSONSerialization isValidJSONObject:ret]) {
        if(errorPtr) {
            NSString * domain = @"com.amazon.TestingService.ErrorDomain";
            NSUInteger errorCode = 0;
            NSMutableDictionary * userInfo = [NSMutableDictionary dictionary];
            [userInfo setObject:@"An error occured in jsonFromVoidResult: the object passed in is not a valid json serializable object." forKey:NSLocalizedDescriptionKey];
            *errorPtr = [NSError errorWithDomain:domain code:errorCode userInfo:userInfo];
        }
        return nil;
    }
    [returnDictionary setObject:ret forKey:@"response"];
    return [NSJSONSerialization dataWithJSONObject:returnDictionary options:0 error:errorPtr];
}

+ (NSData *)jsonFromBooleanResult:(AMAZONBooleanResult *)booleanResult uuid:(NSString *)uuid error:(NSError *__autoreleasing *)errorPtr {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    [returnDictionary setObject:uuid forKey:@"callerId"];
    NSDictionary* ret = [booleanResult dictionaryValue];
    if(![NSJSONSerialization isValidJSONObject:ret]) {
        if(errorPtr) {
            NSString * domain = @"com.amazon.TestingService.ErrorDomain";
            NSUInteger errorCode = 0;
            NSMutableDictionary * userInfo = [NSMutableDictionary dictionary];
            [userInfo setObject:@"An error occured in jsonFromBooleanResult: the object passed in is not a valid json serializable object." forKey:NSLocalizedDescriptionKey];
            *errorPtr = [NSError errorWithDomain:domain code:errorCode userInfo:userInfo];
        }
        return nil;
    }
    [returnDictionary setObject:ret forKey:@"response"];
    return [NSJSONSerialization dataWithJSONObject:returnDictionary options:0 error:errorPtr];
}

+ (NSData *)jsonFromLongResult:(AMAZONLongResult *)longResult uuid:(NSString *)uuid error:(NSError *__autoreleasing *)errorPtr {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    [returnDictionary setObject:uuid forKey:@"callerId"];
    NSDictionary* ret = [longResult dictionaryValue];
    if(![NSJSONSerialization isValidJSONObject:ret]) {
        if(errorPtr) {
            NSString * domain = @"com.amazon.TestingService.ErrorDomain";
            NSUInteger errorCode = 0;
            NSMutableDictionary * userInfo = [NSMutableDictionary dictionary];
            [userInfo setObject:@"An error occured in jsonFromLongResult: the object passed in is not a valid json serializable object." forKey:NSLocalizedDescriptionKey];
            *errorPtr = [NSError errorWithDomain:domain code:errorCode userInfo:userInfo];
        }
        return nil;
    }
    [returnDictionary setObject:ret forKey:@"response"];
    return [NSJSONSerialization dataWithJSONObject:returnDictionary options:0 error:errorPtr];
}

+ (NSData *)jsonFromStatus:(AMAZONStatus *)status uuid:(NSString *)uuid error:(NSError *__autoreleasing *)errorPtr {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    [returnDictionary setObject:uuid forKey:@"callerId"];
    NSDictionary* ret = [status dictionaryValue];
    if(![NSJSONSerialization isValidJSONObject:ret]) {
        if(errorPtr) {
            NSString * domain = @"com.amazon.TestingService.ErrorDomain";
            NSUInteger errorCode = 0;
            NSMutableDictionary * userInfo = [NSMutableDictionary dictionary];
            [userInfo setObject:@"An error occured in jsonFromStatus: the object passed in is not a valid json serializable object." forKey:NSLocalizedDescriptionKey];
            *errorPtr = [NSError errorWithDomain:domain code:errorCode userInfo:userInfo];
        }
        return nil;
    }
    [returnDictionary setObject:ret forKey:@"response"];
    return [NSJSONSerialization dataWithJSONObject:returnDictionary options:0 error:errorPtr];
}

+ (NSData *)jsonFromPlayerInfo:(AMAZONPlayerInfo *)playerInfo uuid:(NSString *)uuid error:(NSError *__autoreleasing *)errorPtr {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    [returnDictionary setObject:uuid forKey:@"callerId"];
    NSDictionary* ret = [playerInfo dictionaryValue];
    if(![NSJSONSerialization isValidJSONObject:ret]) {
        if(errorPtr) {
            NSString * domain = @"com.amazon.TestingService.ErrorDomain";
            NSUInteger errorCode = 0;
            NSMutableDictionary * userInfo = [NSMutableDictionary dictionary];
            [userInfo setObject:@"An error occured in jsonFromPlayerInfo: the object passed in is not a valid json serializable object." forKey:NSLocalizedDescriptionKey];
            *errorPtr = [NSError errorWithDomain:domain code:errorCode userInfo:userInfo];
        }
        return nil;
    }
    [returnDictionary setObject:ret forKey:@"response"];
    return [NSJSONSerialization dataWithJSONObject:returnDictionary options:0 error:errorPtr];
}

+ (NSData *)jsonFromPlayerEvent:(AMAZONPlayer *)player error:(NSError *__autoreleasing *)errorPtr {
    NSMutableDictionary* returnDictionary = [NSMutableDictionary dictionary];
    NSDictionary* ret = [player dictionaryValue];
    if(![NSJSONSerialization isValidJSONObject:ret]) {
        if(errorPtr) {
            NSString * domain = @"com.amazon.TestingService.ErrorDomain";
            NSUInteger errorCode = 0;
            NSMutableDictionary * userInfo = [NSMutableDictionary dictionary];
            [userInfo setObject:@"An error occured in jsonFromPlayer: the object passed in is not a valid json serializable object." forKey:NSLocalizedDescriptionKey];
            *errorPtr = [NSError errorWithDomain:domain code:errorCode userInfo:userInfo];
        }
        return nil;
    }
    [returnDictionary setObject:ret forKey:@"response"];
    [returnDictionary setObject:@"playerDiscovered" forKey:@"eventId"];
    return [NSJSONSerialization dataWithJSONObject:returnDictionary options:0 error:errorPtr];
}

+ (NSData *)jsonFromOnStatusChangeParametersEvent:(AMAZONOnStatusChangeParameters *)onStatusChangeParameters error:(NSError *__autoreleasing *)errorPtr {
    NSMutableDictionary* returnDictionary = [NSMutableDictionary dictionary];
    NSDictionary* ret = [onStatusChangeParameters dictionaryValue];
    if(![NSJSONSerialization isValidJSONObject:ret]) {
        if(errorPtr) {
            NSString * domain = @"com.amazon.TestingService.ErrorDomain";
            NSUInteger errorCode = 0;
            NSMutableDictionary * userInfo = [NSMutableDictionary dictionary];
            [userInfo setObject:@"An error occured in jsonFromOnStatusChangeParameters: the object passed in is not a valid json serializable object." forKey:NSLocalizedDescriptionKey];
            *errorPtr = [NSError errorWithDomain:domain code:errorCode userInfo:userInfo];
        }
        return nil;
    }
    [returnDictionary setObject:ret forKey:@"response"];
    [returnDictionary setObject:@"onStatusChange" forKey:@"eventId"];
    return [NSJSONSerialization dataWithJSONObject:returnDictionary options:0 error:errorPtr];
}

@end

