
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

//
//  AMAZONAmazonFlingServiceObjectFromJson.m
//


#import "AMAZONAmazonFlingServiceObjectFromJson.h"
#import "AMAZONPlayerServiceId.h"
#import "AMAZONPlayer.h"
#import "AMAZONSetVolumeParameters.h"
#import "AMAZONMuteParameters.h"
#import "AMAZONIsMimeTypeSupportedParameters.h"
#import "AMAZONPlayerSeekParameters.h"
#import "AMAZONSetMediaSourceParameters.h"
#import "AMAZONSetPlayerStyleParameters.h"
#import "AMAZONSetPositionUpdateIntervalParameters.h"
#import "AMAZONSendCommandParameters.h"

@interface AMAZONAmazonFlingServiceObjectFromJson ()

+ (NSDictionary *)dictionaryWrapperFromJson:(NSData *)jsonData error:(NSError *__autoreleasing *)errorPtr;
+ (NSArray *)arrayWrapperFromJson:(NSData *)jsonData error:(NSError *__autoreleasing *)errorPtr;

@end

@implementation AMAZONAmazonFlingServiceObjectFromJson

+ (NSDictionary *)dictionaryWrapperFromJson:(NSData *)jsonData error:(NSError *__autoreleasing *)errorPtr {
    NSObject *arg = [NSJSONSerialization JSONObjectWithData:jsonData options:0 error:errorPtr];
    return (NSDictionary *)arg;
}

+ (NSArray *)arrayWrapperFromJson:(NSData *)jsonData error:(NSError *__autoreleasing *)errorPtr {
    NSObject *arg = [NSJSONSerialization JSONObjectWithData:jsonData options:0 error:errorPtr];
    return (NSArray *)arg;
}

+ (AMAZONPlayerServiceId *)playerServiceIdFromJson:(NSData *)jsonData error:(NSError *__autoreleasing *)errorPtr {
    NSError * error = nil;
    NSDictionary *dict = [AMAZONAmazonFlingServiceObjectFromJson dictionaryWrapperFromJson:jsonData error:&error];
    if(dict == nil && error) {
        if(errorPtr) {
            *errorPtr = error;
        }
        return nil;
    }
    AMAZONPlayerServiceId * returnPlayerServiceId = [[AMAZONPlayerServiceId alloc] initWithDictionary:dict];
    return returnPlayerServiceId;
}

+ (AMAZONPlayer *)playerFromJson:(NSData *)jsonData error:(NSError *__autoreleasing *)errorPtr {
    NSError * error = nil;
    NSDictionary *dict = [AMAZONAmazonFlingServiceObjectFromJson dictionaryWrapperFromJson:jsonData error:&error];
    if(dict == nil && error) {
        if(errorPtr) {
            *errorPtr = error;
        }
        return nil;
    }
    AMAZONPlayer * returnPlayer = [[AMAZONPlayer alloc] initWithDictionary:dict];
    return returnPlayer;
}

+ (AMAZONSetVolumeParameters *)setVolumeParametersFromJson:(NSData *)jsonData error:(NSError *__autoreleasing *)errorPtr {
    NSError * error = nil;
    NSDictionary *dict = [AMAZONAmazonFlingServiceObjectFromJson dictionaryWrapperFromJson:jsonData error:&error];
    if(dict == nil && error) {
        if(errorPtr) {
            *errorPtr = error;
        }
        return nil;
    }
    AMAZONSetVolumeParameters * returnSetVolumeParameters = [[AMAZONSetVolumeParameters alloc] initWithDictionary:dict];
    return returnSetVolumeParameters;
}

+ (AMAZONMuteParameters *)muteParametersFromJson:(NSData *)jsonData error:(NSError *__autoreleasing *)errorPtr {
    NSError * error = nil;
    NSDictionary *dict = [AMAZONAmazonFlingServiceObjectFromJson dictionaryWrapperFromJson:jsonData error:&error];
    if(dict == nil && error) {
        if(errorPtr) {
            *errorPtr = error;
        }
        return nil;
    }
    AMAZONMuteParameters * returnMuteParameters = [[AMAZONMuteParameters alloc] initWithDictionary:dict];
    return returnMuteParameters;
}

+ (AMAZONIsMimeTypeSupportedParameters *)isMimeTypeSupportedParametersFromJson:(NSData *)jsonData error:(NSError *__autoreleasing *)errorPtr {
    NSError * error = nil;
    NSDictionary *dict = [AMAZONAmazonFlingServiceObjectFromJson dictionaryWrapperFromJson:jsonData error:&error];
    if(dict == nil && error) {
        if(errorPtr) {
            *errorPtr = error;
        }
        return nil;
    }
    AMAZONIsMimeTypeSupportedParameters * returnIsMimeTypeSupportedParameters = [[AMAZONIsMimeTypeSupportedParameters alloc] initWithDictionary:dict];
    return returnIsMimeTypeSupportedParameters;
}

+ (AMAZONPlayerSeekParameters *)playerSeekParametersFromJson:(NSData *)jsonData error:(NSError *__autoreleasing *)errorPtr {
    NSError * error = nil;
    NSDictionary *dict = [AMAZONAmazonFlingServiceObjectFromJson dictionaryWrapperFromJson:jsonData error:&error];
    if(dict == nil && error) {
        if(errorPtr) {
            *errorPtr = error;
        }
        return nil;
    }
    AMAZONPlayerSeekParameters * returnPlayerSeekParameters = [[AMAZONPlayerSeekParameters alloc] initWithDictionary:dict];
    return returnPlayerSeekParameters;
}

+ (AMAZONSetMediaSourceParameters *)setMediaSourceParametersFromJson:(NSData *)jsonData error:(NSError *__autoreleasing *)errorPtr {
    NSError * error = nil;
    NSDictionary *dict = [AMAZONAmazonFlingServiceObjectFromJson dictionaryWrapperFromJson:jsonData error:&error];
    if(dict == nil && error) {
        if(errorPtr) {
            *errorPtr = error;
        }
        return nil;
    }
    AMAZONSetMediaSourceParameters * returnSetMediaSourceParameters = [[AMAZONSetMediaSourceParameters alloc] initWithDictionary:dict];
    return returnSetMediaSourceParameters;
}

+ (AMAZONSetPlayerStyleParameters *)setPlayerStyleParametersFromJson:(NSData *)jsonData error:(NSError *__autoreleasing *)errorPtr {
    NSError * error = nil;
    NSDictionary *dict = [AMAZONAmazonFlingServiceObjectFromJson dictionaryWrapperFromJson:jsonData error:&error];
    if(dict == nil && error) {
        if(errorPtr) {
            *errorPtr = error;
        }
        return nil;
    }
    AMAZONSetPlayerStyleParameters * returnSetPlayerStyleParameters = [[AMAZONSetPlayerStyleParameters alloc] initWithDictionary:dict];
    return returnSetPlayerStyleParameters;
}

+ (AMAZONSetPositionUpdateIntervalParameters *)setPositionUpdateIntervalParametersFromJson:(NSData *)jsonData error:(NSError *__autoreleasing *)errorPtr {
    NSError * error = nil;
    NSDictionary *dict = [AMAZONAmazonFlingServiceObjectFromJson dictionaryWrapperFromJson:jsonData error:&error];
    if(dict == nil && error) {
        if(errorPtr) {
            *errorPtr = error;
        }
        return nil;
    }
    AMAZONSetPositionUpdateIntervalParameters * returnSetPositionUpdateIntervalParameters = [[AMAZONSetPositionUpdateIntervalParameters alloc] initWithDictionary:dict];
    return returnSetPositionUpdateIntervalParameters;
}

+ (AMAZONSendCommandParameters *)sendCommandParametersFromJson:(NSData *)jsonData error:(NSError *__autoreleasing *)errorPtr {
    NSError * error = nil;
    NSDictionary *dict = [AMAZONAmazonFlingServiceObjectFromJson dictionaryWrapperFromJson:jsonData error:&error];
    if(dict == nil && error) {
        if(errorPtr) {
            *errorPtr = error;
        }
        return nil;
    }
    AMAZONSendCommandParameters * returnSendCommandParameters = [[AMAZONSendCommandParameters alloc] initWithDictionary:dict];
    return returnSendCommandParameters;
}

@end

