
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

//
//  AMAZONAmazonFlingServiceObjectiveCController.h
//

#import "AMAZONAmazonFlingService.h"
#import <Foundation/Foundation.h>
#import "UIKit/UIKit.h"
#import "AMAZONPlayer.h"
#import "AMAZONOnStatusChangeParameters.h"

@protocol AMAZONAmazonFlingService;

@protocol AMAZONAmazonFlingServiceEventListenerDelegate <NSObject>

@required
- (void)fireSDKEvent:(NSString *)eventJsonString;

@end

typedef void (^futureJson)(NSData* jsonData);
typedef void (*callback_delegate)(const char*);

@protocol AMAZONAmazonFlingServiceObjectiveCController <NSObject>

@property (atomic, strong) id<AMAZONAmazonFlingService> amazonFlingService;
@property (atomic, copy) NSString* callbackGameObject;
@property (atomic, copy) NSString* crossPlatformTool;
@property (nonatomic, strong) id eventListenerDelegate;
@property (atomic) callback_delegate eventApiMethod;

+ (id<AMAZONAmazonFlingServiceObjectiveCController>)sharedInstance;

- (NSData *)startDiscovery:(NSData *)jsonMsg;
- (NSData *)stopDiscovery:(NSData *)jsonMsg;
- (NSData *)getVolume:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)setVolume:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)isMute:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)setMute:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)getPosition:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)getDuration:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)getStatus:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)isMimeTypeSupported:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)pause:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)play:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)stop:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)seek:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)setMediaSource:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)setPlayerStyle:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)addStatusListener:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)removeStatusListener:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)setPositionUpdateInterval:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)sendCommand:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;
- (NSData *)getMediaInfo:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)completion;

- (void)firePlayerDiscovered:(AMAZONPlayer *)player;
- (void)firePlayerLost:(AMAZONPlayer *)player;
- (void)fireDiscoveryFailure;
- (void)fireOnStatusChange:(AMAZONOnStatusChangeParameters *)onStatusChangeParameters;
- (UIViewController *)currentViewController;
- (NSString *)getCrossPlatformTool;

@end
