
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

//
//  AMAZONAmazonFlingServiceObjectiveCControllerImpl.m
//


#import "AMAZONAmazonFlingServiceObjectiveCControllerImpl.h"
#import "AMAZONAmazonFlingServiceJsonFromObject.h"
#import "AMAZONAmazonFlingServiceObjectFromJson.h"
#import "AMAZONAmazonFlingServiceImpl.h"
#import "UIKit/UIKit.h"

@implementation AMAZONAmazonFlingServiceObjectiveCControllerImpl

@synthesize amazonFlingService = _amazonFlingService;
@synthesize callbackGameObject = _callbackGameObject;
@synthesize crossPlatformTool = _crossPlatformTool;
@synthesize eventListenerDelegate = _eventListenerDelegate;
@synthesize eventApiMethod = _eventApiMethod;

+ (AMAZONAmazonFlingServiceObjectiveCControllerImpl *)sharedInstance
{
    static AMAZONAmazonFlingServiceObjectiveCControllerImpl* sharedController = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedController = [[AMAZONAmazonFlingServiceObjectiveCControllerImpl alloc] init];
    });
    return sharedController;
}

- (instancetype)init
{
    self = [super init];
    if (self) {
        _amazonFlingService = [[AMAZONAmazonFlingServiceImpl alloc] init];
        _amazonFlingService.controller = self;
    }
    return self;
}

- (NSData *)startDiscovery:(NSData *)jsonMsg {
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONPlayerServiceId * playerServiceId = [AMAZONAmazonFlingServiceObjectFromJson playerServiceIdFromJson:jsonMsg error:&objectFromJsonError];
    if(playerServiceId == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, and return the output
    NSError * nativeFunctionError = nil;
    [self.amazonFlingService startDiscovery:playerServiceId error:&nativeFunctionError];
    if(nativeFunctionError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:nativeFunctionError];
    }
    return [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
}

- (NSData *)stopDiscovery:(NSData *)jsonMsg {
    // Call the native function, and return the output
    NSError * nativeFunctionError = nil;
    [self.amazonFlingService stopDiscoveryWithError:&nativeFunctionError];
    if(nativeFunctionError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:nativeFunctionError];
    }
    return [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
}

- (NSData *)getVolume:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONPlayer * player = [AMAZONAmazonFlingServiceObjectFromJson playerFromJson:jsonMsg error:&objectFromJsonError];
    if(player == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService getVolume:player completion:^(AMAZONDoubleResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromDoubleResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)setVolume:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONSetVolumeParameters * setVolumeParameters = [AMAZONAmazonFlingServiceObjectFromJson setVolumeParametersFromJson:jsonMsg error:&objectFromJsonError];
    if(setVolumeParameters == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService setVolume:setVolumeParameters completion:^(AMAZONVoidResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoidResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)isMute:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONPlayer * player = [AMAZONAmazonFlingServiceObjectFromJson playerFromJson:jsonMsg error:&objectFromJsonError];
    if(player == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService isMute:player completion:^(AMAZONBooleanResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromBooleanResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)setMute:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONMuteParameters * muteParameters = [AMAZONAmazonFlingServiceObjectFromJson muteParametersFromJson:jsonMsg error:&objectFromJsonError];
    if(muteParameters == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService setMute:muteParameters completion:^(AMAZONVoidResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoidResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)getPosition:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONPlayer * player = [AMAZONAmazonFlingServiceObjectFromJson playerFromJson:jsonMsg error:&objectFromJsonError];
    if(player == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService getPosition:player completion:^(AMAZONLongResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromLongResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)getDuration:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONPlayer * player = [AMAZONAmazonFlingServiceObjectFromJson playerFromJson:jsonMsg error:&objectFromJsonError];
    if(player == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService getDuration:player completion:^(AMAZONLongResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromLongResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)getStatus:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONPlayer * player = [AMAZONAmazonFlingServiceObjectFromJson playerFromJson:jsonMsg error:&objectFromJsonError];
    if(player == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService getStatus:player completion:^(AMAZONStatus * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromStatus:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)isMimeTypeSupported:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONIsMimeTypeSupportedParameters * isMimeTypeSupportedParameters = [AMAZONAmazonFlingServiceObjectFromJson isMimeTypeSupportedParametersFromJson:jsonMsg error:&objectFromJsonError];
    if(isMimeTypeSupportedParameters == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService isMimeTypeSupported:isMimeTypeSupportedParameters completion:^(AMAZONBooleanResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromBooleanResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)pause:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONPlayer * player = [AMAZONAmazonFlingServiceObjectFromJson playerFromJson:jsonMsg error:&objectFromJsonError];
    if(player == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService pause:player completion:^(AMAZONVoidResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoidResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)play:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONPlayer * player = [AMAZONAmazonFlingServiceObjectFromJson playerFromJson:jsonMsg error:&objectFromJsonError];
    if(player == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService play:player completion:^(AMAZONVoidResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoidResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)stop:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONPlayer * player = [AMAZONAmazonFlingServiceObjectFromJson playerFromJson:jsonMsg error:&objectFromJsonError];
    if(player == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService stop:player completion:^(AMAZONVoidResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoidResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)seek:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONPlayerSeekParameters * playerSeekParameters = [AMAZONAmazonFlingServiceObjectFromJson playerSeekParametersFromJson:jsonMsg error:&objectFromJsonError];
    if(playerSeekParameters == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService seek:playerSeekParameters completion:^(AMAZONVoidResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoidResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)setMediaSource:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONSetMediaSourceParameters * setMediaSourceParameters = [AMAZONAmazonFlingServiceObjectFromJson setMediaSourceParametersFromJson:jsonMsg error:&objectFromJsonError];
    if(setMediaSourceParameters == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService setMediaSource:setMediaSourceParameters completion:^(AMAZONVoidResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoidResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)setPlayerStyle:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONSetPlayerStyleParameters * setPlayerStyleParameters = [AMAZONAmazonFlingServiceObjectFromJson setPlayerStyleParametersFromJson:jsonMsg error:&objectFromJsonError];
    if(setPlayerStyleParameters == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService setPlayerStyle:setPlayerStyleParameters completion:^(AMAZONVoidResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoidResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)addStatusListener:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONPlayer * player = [AMAZONAmazonFlingServiceObjectFromJson playerFromJson:jsonMsg error:&objectFromJsonError];
    if(player == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService addStatusListener:player completion:^(AMAZONVoidResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoidResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)removeStatusListener:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONPlayer * player = [AMAZONAmazonFlingServiceObjectFromJson playerFromJson:jsonMsg error:&objectFromJsonError];
    if(player == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService removeStatusListener:player completion:^(AMAZONVoidResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoidResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)setPositionUpdateInterval:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONSetPositionUpdateIntervalParameters * setPositionUpdateIntervalParameters = [AMAZONAmazonFlingServiceObjectFromJson setPositionUpdateIntervalParametersFromJson:jsonMsg error:&objectFromJsonError];
    if(setPositionUpdateIntervalParameters == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService setPositionUpdateInterval:setPositionUpdateIntervalParameters completion:^(AMAZONVoidResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoidResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)sendCommand:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONSendCommandParameters * sendCommandParameters = [AMAZONAmazonFlingServiceObjectFromJson sendCommandParametersFromJson:jsonMsg error:&objectFromJsonError];
    if(sendCommandParameters == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService sendCommand:sendCommandParameters completion:^(AMAZONVoidResult * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoidResult:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (NSData *)getMediaInfo:(NSData *)jsonMsg uuid:(NSString *)uuid completion:(futureJson)responseJson {
    // Create the returning JSON that has just the UUID
    NSData* returnJson = [AMAZONAmazonFlingServiceJsonFromObject jsonFromVoid];
    // Get the input argument from the JSON
    NSError * objectFromJsonError = nil;
    AMAZONPlayer * player = [AMAZONAmazonFlingServiceObjectFromJson playerFromJson:jsonMsg error:&objectFromJsonError];
    if(player == nil && objectFromJsonError) {
        return [AMAZONAmazonFlingServiceJsonFromObject jsonFromError:objectFromJsonError];
    }
    // Pass the input argument into the native function, which will put the completed future object in the responseJson on completion
    [self.amazonFlingService getMediaInfo:player completion:^(AMAZONPlayerInfo * response, NSError * error)
    {
        if(response == nil && error) {
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:error uuid:uuid];
            responseJson(returnJsonFuture);
            return;
        } else {
            NSError * jsonFromObjectError = nil;
            NSData* returnJsonFuture = [AMAZONAmazonFlingServiceJsonFromObject jsonFromPlayerInfo:response uuid:uuid error:&jsonFromObjectError];
            if(returnJsonFuture == nil && jsonFromObjectError) {
                responseJson([AMAZONAmazonFlingServiceJsonFromObject jsonFromFutureError:jsonFromObjectError uuid:uuid]);
                return;
            }
            responseJson(returnJsonFuture);
            return;
        }
    }];
    
    return returnJson;
}

- (void)firePlayerDiscovered:(AMAZONPlayer *)player {
    NSError * jsonFromObjectError = nil;
    NSData* jsonData = [AMAZONAmazonFlingServiceJsonFromObject jsonFromPlayerEvent:player error:&jsonFromObjectError];
    if(jsonData == nil && jsonFromObjectError) {
        jsonData = [AMAZONAmazonFlingServiceJsonFromObject jsonFromEventError:jsonFromObjectError eventId:@"playerDiscovered"];
    }
    NSString* jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    if(self.eventListenerDelegate && [self.eventListenerDelegate respondsToSelector:@selector(fireSDKEvent:)]) {
        [self.eventListenerDelegate fireSDKEvent:jsonString];
    } else {
#ifdef UNITY_VERSION
        UnitySendMessage([self.callbackGameObject UTF8String], "UnityFireEvent", [jsonString UTF8String]);
#else
        if(self.eventApiMethod){
            self.eventApiMethod([jsonString UTF8String]);
        }
#endif
    }
}

- (void)firePlayerLost:(AMAZONPlayer *)player {
    NSError * jsonFromObjectError = nil;
    NSData* jsonData = [AMAZONAmazonFlingServiceJsonFromObject jsonFromPlayerEvent:player error:&jsonFromObjectError];
    if(jsonData == nil && jsonFromObjectError) {
        jsonData = [AMAZONAmazonFlingServiceJsonFromObject jsonFromEventError:jsonFromObjectError eventId:@"playerLost"];
    }
    NSString* jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    if(self.eventListenerDelegate && [self.eventListenerDelegate respondsToSelector:@selector(fireSDKEvent:)]) {
        [self.eventListenerDelegate fireSDKEvent:jsonString];
    } else {
#ifdef UNITY_VERSION
        UnitySendMessage([self.callbackGameObject UTF8String], "UnityFireEvent", [jsonString UTF8String]);
#else
        if(self.eventApiMethod){
            self.eventApiMethod([jsonString UTF8String]);
        }
#endif
    }
}

- (void)fireDiscoveryFailure {
    NSData* jsonData = [AMAZONAmazonFlingServiceJsonFromObject jsonFromEventNoOutput:@"discoveryFailure"];
    NSString* jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    if(self.eventListenerDelegate && [self.eventListenerDelegate respondsToSelector:@selector(fireSDKEvent:)]) {
        [self.eventListenerDelegate fireSDKEvent:jsonString];
    } else {
#ifdef UNITY_VERSION
        UnitySendMessage([self.callbackGameObject UTF8String], "UnityFireEvent", [jsonString UTF8String]);
#else
        if(self.eventApiMethod){
            self.eventApiMethod([jsonString UTF8String]);
        }
#endif
    }
}

- (void)fireOnStatusChange:(AMAZONOnStatusChangeParameters *)onStatusChangeParameters {
    NSError * jsonFromObjectError = nil;
    NSData* jsonData = [AMAZONAmazonFlingServiceJsonFromObject jsonFromOnStatusChangeParametersEvent:onStatusChangeParameters error:&jsonFromObjectError];
    if(jsonData == nil && jsonFromObjectError) {
        jsonData = [AMAZONAmazonFlingServiceJsonFromObject jsonFromEventError:jsonFromObjectError eventId:@"onStatusChange"];
    }
    NSString* jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    if(self.eventListenerDelegate && [self.eventListenerDelegate respondsToSelector:@selector(fireSDKEvent:)]) {
        [self.eventListenerDelegate fireSDKEvent:jsonString];
    } else {
#ifdef UNITY_VERSION
        UnitySendMessage([self.callbackGameObject UTF8String], "UnityFireEvent", [jsonString UTF8String]);
#else
        if(self.eventApiMethod){
            self.eventApiMethod([jsonString UTF8String]);
        }
#endif
    }
}


-(UIViewController*) findBestViewController:(UIViewController*) vc {
    if (vc.presentedViewController) {
        // Return presented view controller
        return [self findBestViewController:vc.presentedViewController];
    } else if ([vc isKindOfClass:[UISplitViewController class]]) {
        // Return right hand side
        UISplitViewController* svc = (UISplitViewController*) vc;
        if (svc.viewControllers.count > 0)
            return [self findBestViewController:svc.viewControllers.lastObject];
        else
            return vc;
    } else if ([vc isKindOfClass:[UINavigationController class]]) {
        // Return top view
        UINavigationController* svc = (UINavigationController*) vc;
        if (svc.viewControllers.count > 0)
            return [self findBestViewController:svc.topViewController];
        else
            return vc;
    } else if ([vc isKindOfClass:[UITabBarController class]]) {
        // Return visible view
        UITabBarController* svc = (UITabBarController*) vc;
        if (svc.viewControllers.count > 0)
            return [self findBestViewController:svc.selectedViewController];
        else
            return vc;
    } else {
        // Base case for recursion: No navigable child view controllers
        return vc;
    }
}

-(UIViewController*) currentViewController {
    // Find best view controller
    UIViewController* viewController = [UIApplication sharedApplication].keyWindow.rootViewController;
    return [self findBestViewController:viewController];
}

-(NSString*) getCrossPlatformTool {
    return self.crossPlatformTool;
}
@end

