
/*
 * Copyright 2014 Amazon.com,
 * Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the
 * "License"). You may not use this file except in compliance
 * with the License. A copy of the License is located at
 *
 * http://aws.amazon.com/apache2.0/
 *
 * or in the "license" file accompanying this file. This file is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and
 * limitations under the License.
 */

//
//  AmazonFlingServiceStatusMonitor.m
//

#import <Foundation/Foundation.h>

#import "AMAZONAmazonFlingServiceStatusMonitor.h"

@implementation AmazonFlingServiceStatusMonitor

- (void) onStatusChange : (MediaPlayerStatus*) status positionChangedTo : (long long) position {
    // Send event to the listener
    AMAZONStatus* sharedStatus = [AMAZONStatus alloc];
    sharedStatus.state = [AmazonFlingServiceStatusMonitor convertState:[status state]];
    sharedStatus.condition = [AmazonFlingServiceStatusMonitor convertCondition:[status condition]];
    sharedStatus.muteSet = [status isMuteSet];
    sharedStatus.mute = [status isMute];
    sharedStatus.volumeSet = [status isVolumeSet];
    sharedStatus.volume = [status volume];
    if (sharedStatus.volume < 0.999) {
        sharedStatus.volume += 0.001;
    }
    AMAZONOnStatusChangeParameters* params = [AMAZONOnStatusChangeParameters alloc];
    params.player = self.player;
    params.status = sharedStatus;
    params.position = position;
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.controller fireOnStatusChange: params];
    });
}

+(AMAZONState) convertState : (enum MediaState) state {
    AMAZONState ret = ERROR;
    switch(state) {
        case NoMedia:
            ret = NOSOURCE;
            break;
        case PreparingMedia:
            ret = PREPARINGMEDIA;
            break;
        case ReadyToPlay:
            ret = READYTOPLAY;
            break;
        case Playing:
            ret = PLAYING;
            break;
        case Paused:
            ret = PAUSED;
            break;
        case Seeking:
            ret = SEEKING;
            break;
        case Finished:
            ret = FINISHED;
            break;
        case Error:
        default:
            // Do nothing
            break;
    }
    
    return ret;
}

+(AMAZONCondition) convertCondition : (enum MediaCondition) condition {
    AMAZONCondition ret = ERRORUNKNOWN;
    switch (condition) {
        case Good:
            ret = GOOD;
            break;
        case WarningContent:
            ret = WARNINGCONTENT;
            break;
        case WarningBandwidth:
            ret = WARNINGBANDWIDTH;
            break;
        case ErrorContent:
            ret = ERRORCONTENT;
            break;
        case ErrorChannel:
            ret = ERRORCHANNEL;
            break;
        case ErrorUnknown:
        default:
            // Do nothing
            break;
    }
    
    return ret;
}

@end

