
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

//
//  AMAZONEnums.m
//


#import <Foundation/Foundation.h>
#import "AMAZONEnums.h"

@implementation AMAZONEnums

NSDictionary * stringToState = nil;
NSDictionary * stateToString = nil;
NSDictionary * stringToCondition = nil;
NSDictionary * conditionToString = nil;
NSDictionary * stringToPlayerSeekMode = nil;
NSDictionary * playerSeekModeToString = nil;

+ (void)initialize {
stringToState = @{
    @"NOSOURCE": [NSNumber numberWithInt:(NOSOURCE)],
    @"PREPARINGMEDIA": [NSNumber numberWithInt:(PREPARINGMEDIA)],
    @"READYTOPLAY": [NSNumber numberWithInt:(READYTOPLAY)],
    @"PLAYING": [NSNumber numberWithInt:(PLAYING)],
    @"PAUSED": [NSNumber numberWithInt:(PAUSED)],
    @"SEEKING": [NSNumber numberWithInt:(SEEKING)],
    @"FINISHED": [NSNumber numberWithInt:(FINISHED)],
    @"ERROR": [NSNumber numberWithInt:(ERROR)]
};

#if !__has_feature(objc_arc)
[stringToState retain];
#endif

stateToString = @{
    [NSNumber numberWithInt:(NOSOURCE)]: @"NOSOURCE",
    [NSNumber numberWithInt:(PREPARINGMEDIA)]: @"PREPARINGMEDIA",
    [NSNumber numberWithInt:(READYTOPLAY)]: @"READYTOPLAY",
    [NSNumber numberWithInt:(PLAYING)]: @"PLAYING",
    [NSNumber numberWithInt:(PAUSED)]: @"PAUSED",
    [NSNumber numberWithInt:(SEEKING)]: @"SEEKING",
    [NSNumber numberWithInt:(FINISHED)]: @"FINISHED",
    [NSNumber numberWithInt:(ERROR)]: @"ERROR"
};

#if !__has_feature(objc_arc)
[stateToString retain];
#endif

stringToCondition = @{
    @"GOOD": [NSNumber numberWithInt:(GOOD)],
    @"WARNINGCONTENT": [NSNumber numberWithInt:(WARNINGCONTENT)],
    @"WARNINGBANDWIDTH": [NSNumber numberWithInt:(WARNINGBANDWIDTH)],
    @"ERRORCONTENT": [NSNumber numberWithInt:(ERRORCONTENT)],
    @"ERRORCHANNEL": [NSNumber numberWithInt:(ERRORCHANNEL)],
    @"ERRORUNKNOWN": [NSNumber numberWithInt:(ERRORUNKNOWN)]
};

#if !__has_feature(objc_arc)
[stringToCondition retain];
#endif

conditionToString = @{
    [NSNumber numberWithInt:(GOOD)]: @"GOOD",
    [NSNumber numberWithInt:(WARNINGCONTENT)]: @"WARNINGCONTENT",
    [NSNumber numberWithInt:(WARNINGBANDWIDTH)]: @"WARNINGBANDWIDTH",
    [NSNumber numberWithInt:(ERRORCONTENT)]: @"ERRORCONTENT",
    [NSNumber numberWithInt:(ERRORCHANNEL)]: @"ERRORCHANNEL",
    [NSNumber numberWithInt:(ERRORUNKNOWN)]: @"ERRORUNKNOWN"
};

#if !__has_feature(objc_arc)
[conditionToString retain];
#endif

stringToPlayerSeekMode = @{
    @"PSM_ABSOLUTE": [NSNumber numberWithInt:(PSM_ABSOLUTE)],
    @"PSM_RELATIVE": [NSNumber numberWithInt:(PSM_RELATIVE)]
};

#if !__has_feature(objc_arc)
[stringToPlayerSeekMode retain];
#endif

playerSeekModeToString = @{
    [NSNumber numberWithInt:(PSM_ABSOLUTE)]: @"PSM_ABSOLUTE",
    [NSNumber numberWithInt:(PSM_RELATIVE)]: @"PSM_RELATIVE"
};

#if !__has_feature(objc_arc)
[playerSeekModeToString retain];
#endif

}

+ (AMAZONState)convertStringToState:(NSString *)stateAsString{
    return [[stringToState objectForKey:(stateAsString)] intValue];
}
+ (NSString *)convertStateToString:(AMAZONState)state{
    return [stateToString objectForKey:([NSNumber numberWithInt:(state)])];
}
+ (AMAZONCondition)convertStringToCondition:(NSString *)conditionAsString{
    return [[stringToCondition objectForKey:(conditionAsString)] intValue];
}
+ (NSString *)convertConditionToString:(AMAZONCondition)condition{
    return [conditionToString objectForKey:([NSNumber numberWithInt:(condition)])];
}
+ (AMAZONPlayerSeekMode)convertStringToPlayerSeekMode:(NSString *)playerSeekModeAsString{
    return [[stringToPlayerSeekMode objectForKey:(playerSeekModeAsString)] intValue];
}
+ (NSString *)convertPlayerSeekModeToString:(AMAZONPlayerSeekMode)playerSeekMode{
    return [playerSeekModeToString objectForKey:([NSNumber numberWithInt:(playerSeekMode)])];
}

@end
