
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

//
//  AMAZONIsMimeTypeSupportedParameters.m
//


#import "AMAZONIsMimeTypeSupportedParameters.h"

@implementation AMAZONIsMimeTypeSupportedParameters

#pragma mark - Lifecycle

- (instancetype)init
{
    self = [super init];
    return self;
}

- (instancetype)initWithDictionary:(NSMutableDictionary *)dict
{
    self = [super init];
    if(self) {
        self.player = [[AMAZONPlayer alloc] initWithDictionary:[dict objectForKey:@"player"]];
        self.mimeType = [dict objectForKey:@"mimeType"];
    }
     
    return self;
}

#pragma mark - Compares

- (NSUInteger)hash
{
    NSUInteger hashCode = 17;
    NSUInteger prime = 31;
    
    hashCode = prime * hashCode + (self.player == NULL ? 0 : [self.player hash]);
    hashCode = prime * hashCode + (self.mimeType == NULL ? 0 : [self.mimeType hash]);

    return hashCode;
}

- (BOOL)isEqual:(id)other
{
    if (other == self) {
        return YES;
    } else if ([other isKindOfClass:[self class]]) {
        return [self compare:(AMAZONIsMimeTypeSupportedParameters*)other] == NSOrderedSame;
    } else {
        return NO;
    }
}

- (NSComparisonResult)compare:(id)other
{
    if (other == NULL) {
        return NSOrderedDescending;
    }
    if (other == self) {
        return NSOrderedSame;
    }
    
    AMAZONIsMimeTypeSupportedParameters* otherAMAZONIsMimeTypeSupportedParameters = other;
    
    if (otherAMAZONIsMimeTypeSupportedParameters.player != self.player) {
        // Null checks
        if (self.player == NULL) {
            return NSOrderedAscending;
        } else if (otherAMAZONIsMimeTypeSupportedParameters.player == NULL) {
            return NSOrderedDescending;
        }
        // Check with isEqual, which checks with compare if it's overriden
        if (![otherAMAZONIsMimeTypeSupportedParameters.player isEqual:self.player]) {
            NSUInteger selfHash = [self hash];
            NSUInteger otherHash = [otherAMAZONIsMimeTypeSupportedParameters hash];
            if(selfHash < otherHash) {
                return NSOrderedAscending;
            } else if(selfHash > otherHash) {
                return NSOrderedDescending;
            }
        }
    }
    
    if (otherAMAZONIsMimeTypeSupportedParameters.mimeType != self.mimeType) {
        // Null checks
        if (self.mimeType == NULL) {
            return NSOrderedAscending;
        } else if (otherAMAZONIsMimeTypeSupportedParameters.mimeType == NULL) {
            return NSOrderedDescending;
        }
        // Check with isEqual, which checks with compare if it's overriden
        if (![otherAMAZONIsMimeTypeSupportedParameters.mimeType isEqual:self.mimeType]) {
            NSUInteger selfHash = [self hash];
            NSUInteger otherHash = [otherAMAZONIsMimeTypeSupportedParameters hash];
            if(selfHash < otherHash) {
                return NSOrderedAscending;
            } else if(selfHash > otherHash) {
                return NSOrderedDescending;
            }
        }
    }
    
    return NSOrderedSame;
}

- (NSDictionary *)dictionaryValue {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    if(self.player == nil) {
        [returnDictionary setObject:[NSNull null] forKey:@"player"];
    } else {
        [returnDictionary setObject:[self.player dictionaryValue] forKey:@"player"];
    }
    if(self.mimeType == nil) {
        [returnDictionary setObject:[NSNull null] forKey:@"mimeType"];
    } else {
        [returnDictionary setObject:self.mimeType forKey:@"mimeType"];
    }
    return returnDictionary;
}

@end
