
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

//
//  AMAZONOnStatusChangeParameters.m
//


#import "AMAZONOnStatusChangeParameters.h"

@implementation AMAZONOnStatusChangeParameters

#pragma mark - Lifecycle

- (instancetype)init
{
    self = [super init];
    return self;
}

- (instancetype)initWithDictionary:(NSMutableDictionary *)dict
{
    self = [super init];
    if(self) {
        self.player = [[AMAZONPlayer alloc] initWithDictionary:[dict objectForKey:@"player"]];
        self.status = [[AMAZONStatus alloc] initWithDictionary:[dict objectForKey:@"status"]];
        self.position = [[dict objectForKey:@"position"] longValue];
    }
     
    return self;
}

#pragma mark - Compares

- (NSUInteger)hash
{
    NSUInteger hashCode = 17;
    NSUInteger prime = 31;
    
    hashCode = prime * hashCode + (self.player == NULL ? 0 : [self.player hash]);
    hashCode = prime * hashCode + (self.status == NULL ? 0 : [self.status hash]);
    hashCode = prime * hashCode + self.position;

    return hashCode;
}

- (BOOL)isEqual:(id)other
{
    if (other == self) {
        return YES;
    } else if ([other isKindOfClass:[self class]]) {
        return [self compare:(AMAZONOnStatusChangeParameters*)other] == NSOrderedSame;
    } else {
        return NO;
    }
}

- (NSComparisonResult)compare:(id)other
{
    if (other == NULL) {
        return NSOrderedDescending;
    }
    if (other == self) {
        return NSOrderedSame;
    }
    
    AMAZONOnStatusChangeParameters* otherAMAZONOnStatusChangeParameters = other;
    
    if (otherAMAZONOnStatusChangeParameters.player != self.player) {
        // Null checks
        if (self.player == NULL) {
            return NSOrderedAscending;
        } else if (otherAMAZONOnStatusChangeParameters.player == NULL) {
            return NSOrderedDescending;
        }
        // Check with isEqual, which checks with compare if it's overriden
        if (![otherAMAZONOnStatusChangeParameters.player isEqual:self.player]) {
            NSUInteger selfHash = [self hash];
            NSUInteger otherHash = [otherAMAZONOnStatusChangeParameters hash];
            if(selfHash < otherHash) {
                return NSOrderedAscending;
            } else if(selfHash > otherHash) {
                return NSOrderedDescending;
            }
        }
    }
    
    if (otherAMAZONOnStatusChangeParameters.status != self.status) {
        // Null checks
        if (self.status == NULL) {
            return NSOrderedAscending;
        } else if (otherAMAZONOnStatusChangeParameters.status == NULL) {
            return NSOrderedDescending;
        }
        // Check with isEqual, which checks with compare if it's overriden
        if (![otherAMAZONOnStatusChangeParameters.status isEqual:self.status]) {
            NSUInteger selfHash = [self hash];
            NSUInteger otherHash = [otherAMAZONOnStatusChangeParameters hash];
            if(selfHash < otherHash) {
                return NSOrderedAscending;
            } else if(selfHash > otherHash) {
                return NSOrderedDescending;
            }
        }
    }
    
    if (otherAMAZONOnStatusChangeParameters.position != self.position) {
        // Null checks
        if (self.position < otherAMAZONOnStatusChangeParameters.position) {
            return NSOrderedAscending;
        } else if (self.position > otherAMAZONOnStatusChangeParameters.position) {
            return NSOrderedDescending;
        }
    }
    
    return NSOrderedSame;
}

- (NSDictionary *)dictionaryValue {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    if(self.player == nil) {
        [returnDictionary setObject:[NSNull null] forKey:@"player"];
    } else {
        [returnDictionary setObject:[self.player dictionaryValue] forKey:@"player"];
    }
    if(self.status == nil) {
        [returnDictionary setObject:[NSNull null] forKey:@"status"];
    } else {
        [returnDictionary setObject:[self.status dictionaryValue] forKey:@"status"];
    }
    [returnDictionary setObject:[NSNumber numberWithLong:self.position] forKey:@"position"];
    return returnDictionary;
}

@end
