
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

//
//  AMAZONPlayer.m
//


#import "AMAZONPlayer.h"

@implementation AMAZONPlayer

#pragma mark - Lifecycle

- (instancetype)init
{
    self = [super init];
    return self;
}

- (instancetype)initWithDictionary:(NSMutableDictionary *)dict
{
    self = [super init];
    if(self) {
        self.playerId = [dict objectForKey:@"playerId"];
        self.playerName = [dict objectForKey:@"playerName"];
    }
     
    return self;
}

#pragma mark - Compares

- (NSUInteger)hash
{
    NSUInteger hashCode = 17;
    NSUInteger prime = 31;
    
    hashCode = prime * hashCode + (self.playerId == NULL ? 0 : [self.playerId hash]);
    hashCode = prime * hashCode + (self.playerName == NULL ? 0 : [self.playerName hash]);

    return hashCode;
}

- (BOOL)isEqual:(id)other
{
    if (other == self) {
        return YES;
    } else if ([other isKindOfClass:[self class]]) {
        return [self compare:(AMAZONPlayer*)other] == NSOrderedSame;
    } else {
        return NO;
    }
}

- (NSComparisonResult)compare:(id)other
{
    if (other == NULL) {
        return NSOrderedDescending;
    }
    if (other == self) {
        return NSOrderedSame;
    }
    
    AMAZONPlayer* otherAMAZONPlayer = other;
    
    if (otherAMAZONPlayer.playerId != self.playerId) {
        // Null checks
        if (self.playerId == NULL) {
            return NSOrderedAscending;
        } else if (otherAMAZONPlayer.playerId == NULL) {
            return NSOrderedDescending;
        }
        // Check with isEqual, which checks with compare if it's overriden
        if (![otherAMAZONPlayer.playerId isEqual:self.playerId]) {
            NSUInteger selfHash = [self hash];
            NSUInteger otherHash = [otherAMAZONPlayer hash];
            if(selfHash < otherHash) {
                return NSOrderedAscending;
            } else if(selfHash > otherHash) {
                return NSOrderedDescending;
            }
        }
    }
    
    if (otherAMAZONPlayer.playerName != self.playerName) {
        // Null checks
        if (self.playerName == NULL) {
            return NSOrderedAscending;
        } else if (otherAMAZONPlayer.playerName == NULL) {
            return NSOrderedDescending;
        }
        // Check with isEqual, which checks with compare if it's overriden
        if (![otherAMAZONPlayer.playerName isEqual:self.playerName]) {
            NSUInteger selfHash = [self hash];
            NSUInteger otherHash = [otherAMAZONPlayer hash];
            if(selfHash < otherHash) {
                return NSOrderedAscending;
            } else if(selfHash > otherHash) {
                return NSOrderedDescending;
            }
        }
    }
    
    return NSOrderedSame;
}

- (NSDictionary *)dictionaryValue {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    if(self.playerId == nil) {
        [returnDictionary setObject:[NSNull null] forKey:@"playerId"];
    } else {
        [returnDictionary setObject:self.playerId forKey:@"playerId"];
    }
    if(self.playerName == nil) {
        [returnDictionary setObject:[NSNull null] forKey:@"playerName"];
    } else {
        [returnDictionary setObject:self.playerName forKey:@"playerName"];
    }
    return returnDictionary;
}

@end
