
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

//
//  AMAZONPlayerInfo.m
//


#import "AMAZONPlayerInfo.h"

@implementation AMAZONPlayerInfo

#pragma mark - Lifecycle

- (instancetype)init
{
    self = [super init];
    return self;
}

- (instancetype)initWithDictionary:(NSMutableDictionary *)dict
{
    self = [super init];
    if(self) {
        self.source = [dict objectForKey:@"source"];
        self.metadataJson = [dict objectForKey:@"metadataJson"];
        self.extra = [dict objectForKey:@"extra"];
    }
     
    return self;
}

#pragma mark - Compares

- (NSUInteger)hash
{
    NSUInteger hashCode = 17;
    NSUInteger prime = 31;
    
    hashCode = prime * hashCode + (self.source == NULL ? 0 : [self.source hash]);
    hashCode = prime * hashCode + (self.metadataJson == NULL ? 0 : [self.metadataJson hash]);
    hashCode = prime * hashCode + (self.extra == NULL ? 0 : [self.extra hash]);

    return hashCode;
}

- (BOOL)isEqual:(id)other
{
    if (other == self) {
        return YES;
    } else if ([other isKindOfClass:[self class]]) {
        return [self compare:(AMAZONPlayerInfo*)other] == NSOrderedSame;
    } else {
        return NO;
    }
}

- (NSComparisonResult)compare:(id)other
{
    if (other == NULL) {
        return NSOrderedDescending;
    }
    if (other == self) {
        return NSOrderedSame;
    }
    
    AMAZONPlayerInfo* otherAMAZONPlayerInfo = other;
    
    if (otherAMAZONPlayerInfo.source != self.source) {
        // Null checks
        if (self.source == NULL) {
            return NSOrderedAscending;
        } else if (otherAMAZONPlayerInfo.source == NULL) {
            return NSOrderedDescending;
        }
        // Check with isEqual, which checks with compare if it's overriden
        if (![otherAMAZONPlayerInfo.source isEqual:self.source]) {
            NSUInteger selfHash = [self hash];
            NSUInteger otherHash = [otherAMAZONPlayerInfo hash];
            if(selfHash < otherHash) {
                return NSOrderedAscending;
            } else if(selfHash > otherHash) {
                return NSOrderedDescending;
            }
        }
    }
    
    if (otherAMAZONPlayerInfo.metadataJson != self.metadataJson) {
        // Null checks
        if (self.metadataJson == NULL) {
            return NSOrderedAscending;
        } else if (otherAMAZONPlayerInfo.metadataJson == NULL) {
            return NSOrderedDescending;
        }
        // Check with isEqual, which checks with compare if it's overriden
        if (![otherAMAZONPlayerInfo.metadataJson isEqual:self.metadataJson]) {
            NSUInteger selfHash = [self hash];
            NSUInteger otherHash = [otherAMAZONPlayerInfo hash];
            if(selfHash < otherHash) {
                return NSOrderedAscending;
            } else if(selfHash > otherHash) {
                return NSOrderedDescending;
            }
        }
    }
    
    if (otherAMAZONPlayerInfo.extra != self.extra) {
        // Null checks
        if (self.extra == NULL) {
            return NSOrderedAscending;
        } else if (otherAMAZONPlayerInfo.extra == NULL) {
            return NSOrderedDescending;
        }
        // Check with isEqual, which checks with compare if it's overriden
        if (![otherAMAZONPlayerInfo.extra isEqual:self.extra]) {
            NSUInteger selfHash = [self hash];
            NSUInteger otherHash = [otherAMAZONPlayerInfo hash];
            if(selfHash < otherHash) {
                return NSOrderedAscending;
            } else if(selfHash > otherHash) {
                return NSOrderedDescending;
            }
        }
    }
    
    return NSOrderedSame;
}

- (NSDictionary *)dictionaryValue {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    if(self.source == nil) {
        [returnDictionary setObject:[NSNull null] forKey:@"source"];
    } else {
        [returnDictionary setObject:self.source forKey:@"source"];
    }
    if(self.metadataJson == nil) {
        [returnDictionary setObject:[NSNull null] forKey:@"metadataJson"];
    } else {
        [returnDictionary setObject:self.metadataJson forKey:@"metadataJson"];
    }
    if(self.extra == nil) {
        [returnDictionary setObject:[NSNull null] forKey:@"extra"];
    } else {
        [returnDictionary setObject:self.extra forKey:@"extra"];
    }
    return returnDictionary;
}

@end
