
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

//
//  AMAZONPlayerServiceId.m
//


#import "AMAZONPlayerServiceId.h"

@implementation AMAZONPlayerServiceId

#pragma mark - Lifecycle

- (instancetype)init
{
    self = [super init];
    return self;
}

- (instancetype)initWithDictionary:(NSMutableDictionary *)dict
{
    self = [super init];
    if(self) {
        self.id = [dict objectForKey:@"id"];
    }
     
    return self;
}

#pragma mark - Compares

- (NSUInteger)hash
{
    NSUInteger hashCode = 17;
    NSUInteger prime = 31;
    
    hashCode = prime * hashCode + (self.id == NULL ? 0 : [self.id hash]);

    return hashCode;
}

- (BOOL)isEqual:(id)other
{
    if (other == self) {
        return YES;
    } else if ([other isKindOfClass:[self class]]) {
        return [self compare:(AMAZONPlayerServiceId*)other] == NSOrderedSame;
    } else {
        return NO;
    }
}

- (NSComparisonResult)compare:(id)other
{
    if (other == NULL) {
        return NSOrderedDescending;
    }
    if (other == self) {
        return NSOrderedSame;
    }
    
    AMAZONPlayerServiceId* otherAMAZONPlayerServiceId = other;
    
    if (otherAMAZONPlayerServiceId.id != self.id) {
        // Null checks
        if (self.id == NULL) {
            return NSOrderedAscending;
        } else if (otherAMAZONPlayerServiceId.id == NULL) {
            return NSOrderedDescending;
        }
        // Check with isEqual, which checks with compare if it's overriden
        if (![otherAMAZONPlayerServiceId.id isEqual:self.id]) {
            NSUInteger selfHash = [self hash];
            NSUInteger otherHash = [otherAMAZONPlayerServiceId hash];
            if(selfHash < otherHash) {
                return NSOrderedAscending;
            } else if(selfHash > otherHash) {
                return NSOrderedDescending;
            }
        }
    }
    
    return NSOrderedSame;
}

- (NSDictionary *)dictionaryValue {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    if(self.id == nil) {
        [returnDictionary setObject:[NSNull null] forKey:@"id"];
    } else {
        [returnDictionary setObject:self.id forKey:@"id"];
    }
    return returnDictionary;
}

@end
