
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

//
//  AMAZONSetMediaSourceParameters.m
//


#import "AMAZONSetMediaSourceParameters.h"

@implementation AMAZONSetMediaSourceParameters

#pragma mark - Lifecycle

- (instancetype)init
{
    self = [super init];
    return self;
}

- (instancetype)initWithDictionary:(NSMutableDictionary *)dict
{
    self = [super init];
    if(self) {
        self.player = [[AMAZONPlayer alloc] initWithDictionary:[dict objectForKey:@"player"]];
        self.source = [dict objectForKey:@"source"];
        self.metadataJson = [dict objectForKey:@"metadataJson"];
        self.autoPlay = [[dict objectForKey:@"autoPlay"] boolValue];
        self.playInBg = [[dict objectForKey:@"playInBg"] boolValue];
    }
     
    return self;
}

#pragma mark - Compares

- (NSUInteger)hash
{
    NSUInteger hashCode = 17;
    NSUInteger prime = 31;
    
    hashCode = prime * hashCode + (self.player == NULL ? 0 : [self.player hash]);
    hashCode = prime * hashCode + (self.source == NULL ? 0 : [self.source hash]);
    hashCode = prime * hashCode + (self.metadataJson == NULL ? 0 : [self.metadataJson hash]);
    hashCode = prime * hashCode + self.autoPlay;
    hashCode = prime * hashCode + self.playInBg;

    return hashCode;
}

- (BOOL)isEqual:(id)other
{
    if (other == self) {
        return YES;
    } else if ([other isKindOfClass:[self class]]) {
        return [self compare:(AMAZONSetMediaSourceParameters*)other] == NSOrderedSame;
    } else {
        return NO;
    }
}

- (NSComparisonResult)compare:(id)other
{
    if (other == NULL) {
        return NSOrderedDescending;
    }
    if (other == self) {
        return NSOrderedSame;
    }
    
    AMAZONSetMediaSourceParameters* otherAMAZONSetMediaSourceParameters = other;
    
    if (otherAMAZONSetMediaSourceParameters.player != self.player) {
        // Null checks
        if (self.player == NULL) {
            return NSOrderedAscending;
        } else if (otherAMAZONSetMediaSourceParameters.player == NULL) {
            return NSOrderedDescending;
        }
        // Check with isEqual, which checks with compare if it's overriden
        if (![otherAMAZONSetMediaSourceParameters.player isEqual:self.player]) {
            NSUInteger selfHash = [self hash];
            NSUInteger otherHash = [otherAMAZONSetMediaSourceParameters hash];
            if(selfHash < otherHash) {
                return NSOrderedAscending;
            } else if(selfHash > otherHash) {
                return NSOrderedDescending;
            }
        }
    }
    
    if (otherAMAZONSetMediaSourceParameters.source != self.source) {
        // Null checks
        if (self.source == NULL) {
            return NSOrderedAscending;
        } else if (otherAMAZONSetMediaSourceParameters.source == NULL) {
            return NSOrderedDescending;
        }
        // Check with isEqual, which checks with compare if it's overriden
        if (![otherAMAZONSetMediaSourceParameters.source isEqual:self.source]) {
            NSUInteger selfHash = [self hash];
            NSUInteger otherHash = [otherAMAZONSetMediaSourceParameters hash];
            if(selfHash < otherHash) {
                return NSOrderedAscending;
            } else if(selfHash > otherHash) {
                return NSOrderedDescending;
            }
        }
    }
    
    if (otherAMAZONSetMediaSourceParameters.metadataJson != self.metadataJson) {
        // Null checks
        if (self.metadataJson == NULL) {
            return NSOrderedAscending;
        } else if (otherAMAZONSetMediaSourceParameters.metadataJson == NULL) {
            return NSOrderedDescending;
        }
        // Check with isEqual, which checks with compare if it's overriden
        if (![otherAMAZONSetMediaSourceParameters.metadataJson isEqual:self.metadataJson]) {
            NSUInteger selfHash = [self hash];
            NSUInteger otherHash = [otherAMAZONSetMediaSourceParameters hash];
            if(selfHash < otherHash) {
                return NSOrderedAscending;
            } else if(selfHash > otherHash) {
                return NSOrderedDescending;
            }
        }
    }
    
    if (otherAMAZONSetMediaSourceParameters.autoPlay != self.autoPlay) {
        // Null checks
        if (self.autoPlay < otherAMAZONSetMediaSourceParameters.autoPlay) {
            return NSOrderedAscending;
        } else if (self.autoPlay > otherAMAZONSetMediaSourceParameters.autoPlay) {
            return NSOrderedDescending;
        }
    }
    
    if (otherAMAZONSetMediaSourceParameters.playInBg != self.playInBg) {
        // Null checks
        if (self.playInBg < otherAMAZONSetMediaSourceParameters.playInBg) {
            return NSOrderedAscending;
        } else if (self.playInBg > otherAMAZONSetMediaSourceParameters.playInBg) {
            return NSOrderedDescending;
        }
    }
    
    return NSOrderedSame;
}

- (NSDictionary *)dictionaryValue {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    if(self.player == nil) {
        [returnDictionary setObject:[NSNull null] forKey:@"player"];
    } else {
        [returnDictionary setObject:[self.player dictionaryValue] forKey:@"player"];
    }
    if(self.source == nil) {
        [returnDictionary setObject:[NSNull null] forKey:@"source"];
    } else {
        [returnDictionary setObject:self.source forKey:@"source"];
    }
    if(self.metadataJson == nil) {
        [returnDictionary setObject:[NSNull null] forKey:@"metadataJson"];
    } else {
        [returnDictionary setObject:self.metadataJson forKey:@"metadataJson"];
    }
    [returnDictionary setObject:[NSNumber numberWithBool:self.autoPlay] forKey:@"autoPlay"];
    [returnDictionary setObject:[NSNumber numberWithBool:self.playInBg] forKey:@"playInBg"];
    return returnDictionary;
}

@end
