
/* 
* Copyright 2014 Amazon.com,
* Inc. or its affiliates. All Rights Reserved.
*
* Licensed under the Apache License, Version 2.0 (the
* "License"). You may not use this file except in compliance
* with the License. A copy of the License is located at
*
* http://aws.amazon.com/apache2.0/
*
* or in the "license" file accompanying this file. This file is
* distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
* CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and
* limitations under the License.
*/

//
//  AMAZONStatus.m
//


#import "AMAZONStatus.h"

@implementation AMAZONStatus

#pragma mark - Lifecycle

- (instancetype)init
{
    self = [super init];
    return self;
}

- (instancetype)initWithDictionary:(NSMutableDictionary *)dict
{
    self = [super init];
    if(self) {
        self.state = [AMAZONEnums convertStringToState:[dict objectForKey:@"state"]];
        self.condition = [AMAZONEnums convertStringToCondition:[dict objectForKey:@"condition"]];
        self.muteSet = [[dict objectForKey:@"muteSet"] boolValue];
        self.mute = [[dict objectForKey:@"mute"] boolValue];
        self.volumeSet = [[dict objectForKey:@"volumeSet"] boolValue];
        self.volume = [[dict objectForKey:@"volume"] doubleValue];
    }
     
    return self;
}

#pragma mark - Compares

- (NSUInteger)hash
{
    NSUInteger hashCode = 17;
    NSUInteger prime = 31;
    
    hashCode = prime * hashCode + self.state;
    hashCode = prime * hashCode + self.condition;
    hashCode = prime * hashCode + self.muteSet;
    hashCode = prime * hashCode + self.mute;
    hashCode = prime * hashCode + self.volumeSet;
    hashCode = prime * hashCode + self.volume;

    return hashCode;
}

- (BOOL)isEqual:(id)other
{
    if (other == self) {
        return YES;
    } else if ([other isKindOfClass:[self class]]) {
        return [self compare:(AMAZONStatus*)other] == NSOrderedSame;
    } else {
        return NO;
    }
}

- (NSComparisonResult)compare:(id)other
{
    if (other == NULL) {
        return NSOrderedDescending;
    }
    if (other == self) {
        return NSOrderedSame;
    }
    
    AMAZONStatus* otherAMAZONStatus = other;
    
    if (otherAMAZONStatus.state != self.state) {
        // Null checks
        if (self.state < otherAMAZONStatus.state) {
            return NSOrderedAscending;
        } else if (self.state > otherAMAZONStatus.state) {
            return NSOrderedDescending;
        }
    }
    
    if (otherAMAZONStatus.condition != self.condition) {
        // Null checks
        if (self.condition < otherAMAZONStatus.condition) {
            return NSOrderedAscending;
        } else if (self.condition > otherAMAZONStatus.condition) {
            return NSOrderedDescending;
        }
    }
    
    if (otherAMAZONStatus.muteSet != self.muteSet) {
        // Null checks
        if (self.muteSet < otherAMAZONStatus.muteSet) {
            return NSOrderedAscending;
        } else if (self.muteSet > otherAMAZONStatus.muteSet) {
            return NSOrderedDescending;
        }
    }
    
    if (otherAMAZONStatus.mute != self.mute) {
        // Null checks
        if (self.mute < otherAMAZONStatus.mute) {
            return NSOrderedAscending;
        } else if (self.mute > otherAMAZONStatus.mute) {
            return NSOrderedDescending;
        }
    }
    
    if (otherAMAZONStatus.volumeSet != self.volumeSet) {
        // Null checks
        if (self.volumeSet < otherAMAZONStatus.volumeSet) {
            return NSOrderedAscending;
        } else if (self.volumeSet > otherAMAZONStatus.volumeSet) {
            return NSOrderedDescending;
        }
    }
    
    if (otherAMAZONStatus.volume != self.volume) {
        // Null checks
        if (self.volume < otherAMAZONStatus.volume) {
            return NSOrderedAscending;
        } else if (self.volume > otherAMAZONStatus.volume) {
            return NSOrderedDescending;
        }
    }
    
    return NSOrderedSame;
}

- (NSDictionary *)dictionaryValue {
    NSMutableDictionary* returnDictionary = [[NSMutableDictionary alloc] init];
    [returnDictionary setObject:[AMAZONEnums convertStateToString:self.state] forKey:@"state"];
    [returnDictionary setObject:[AMAZONEnums convertConditionToString:self.condition] forKey:@"condition"];
    [returnDictionary setObject:[NSNumber numberWithBool:self.muteSet] forKey:@"muteSet"];
    [returnDictionary setObject:[NSNumber numberWithBool:self.mute] forKey:@"mute"];
    [returnDictionary setObject:[NSNumber numberWithBool:self.volumeSet] forKey:@"volumeSet"];
    [returnDictionary setObject:[NSNumber numberWithDouble:self.volume] forKey:@"volume"];
    return returnDictionary;
}

@end
